@@ -16,26 +16,49 @@
 
 //  See http://www.boost.org for updates, documentation, and revision history.
 
+#include <cassert>
+#include <cstddef> // NULL
+
 #include <boost/noncopyable.hpp>
+#include <boost/config.hpp>
 #include <boost/serialization/extended_type_info.hpp>
 
-namespace boost { 
+namespace boost {
 namespace archive {
 namespace detail {
 
-class basic_serializer : private boost::noncopyable
+class basic_serializer : 
+    private boost::noncopyable
 {
+    const boost::serialization::extended_type_info * m_eti;
 protected:
     explicit basic_serializer(
-        const boost::serialization::extended_type_info & type_
+        const boost::serialization::extended_type_info & eti
     ) : 
-        type(type_)
-    {}
-public:
-    const boost::serialization::extended_type_info & type;
-    bool operator<(const basic_serializer & rhs) const {
-        return type < rhs.type;
+        m_eti(& eti)
+    {
+        assert(NULL != & eti);
     }
+public:
+    const boost::serialization::extended_type_info & get_eti() const {
+        return * m_eti;
+    }
+};
+
+inline bool 
+operator<(const basic_serializer & lhs, const basic_serializer & rhs)  {
+    // can't compare address since there can be multiple eti records
+    // for the same type in different execution modules (that is, DLLS)
+    // leave this here as a reminder not to do this!
+    // return & lhs.get_eti() < & rhs.get_eti();
+    return lhs.get_eti() < rhs.get_eti();
+}
+
+class basic_serializer_arg : public basic_serializer {
+public:
+    basic_serializer_arg(const serialization::extended_type_info & eti) :
+        basic_serializer(eti)
+    {}
 };
 
 } // namespace detail
