@@ -1,5 +1,5 @@
-#ifndef  BOOST_TYPEINFO_EXTENDED_MAP_HPP
-#define BOOST_TYPEINFO_EXTENDED_MAP_HPP
+#ifndef  BOOST_SERIALIZER_MAP_HPP
+#define BOOST_SERIALIZER_MAP_HPP
 
 // MS compatible compilers support #pragma once
 #if defined(_MSC_VER) && (_MSC_VER >= 1020)
@@ -16,7 +16,13 @@
 
 //  See http://www.boost.org for updates, documentation, and revision history.
 
-#include <set>
+#include <list>
+
+#include <boost/config.hpp>
+#include <boost/archive/detail/auto_link_archive.hpp>
+#include <boost/archive/detail/abi_prefix.hpp> // must be the last header
+
+#include <boost/utility.hpp>
 
 namespace boost { 
 namespace serialization {
@@ -24,31 +30,30 @@ namespace serialization {
 }
 
 namespace archive {
-namespace detail  {
+namespace detail {
 
 class basic_serializer;
 
-struct type_info_pointer_compare
+class BOOST_ARCHIVE_OR_WARCHIVE_DECL(BOOST_PP_EMPTY()) basic_serializer_map : public
+    boost::noncopyable
 {
-    bool operator()(
-        const basic_serializer * lhs, const basic_serializer * rhs
-    ) const    {
-        return *lhs < *rhs;
-    }
-};
-
-struct basic_serializer_map
-{
-    typedef std::set<const basic_serializer *, type_info_pointer_compare> map_type;
-    map_type map;
-    bool insert(const basic_serializer * bs);
-    const basic_serializer * tfind(
+    typedef std::list<const basic_serializer *> map_type;
+    map_type m_map;
+public:
+    void insert(const basic_serializer * bs);
+    void erase(const basic_serializer * bs);
+    const basic_serializer * find(
         const boost::serialization::extended_type_info & type_
     ) const;
+private:
+    // cw 8.3 requires this
+    basic_serializer_map& operator=(basic_serializer_map const&);
 };
 
 } // namespace detail
 } // namespace archive
 } // namespace boost
 
-#endif // BOOST_TYPEINFO_EXTENDED_MAP_HPP
+#include <boost/archive/detail/abi_suffix.hpp> // must be the last header
+
+#endif // BOOST_SERIALIZER_MAP_HPP
