@@ -18,17 +18,17 @@
 
 #include <boost/config.hpp>
 #include <boost/archive/binary_iarchive.hpp>
-#include <boost/archive/detail/polymorphic_iarchive_dispatch.hpp>
+#include <boost/archive/detail/polymorphic_iarchive_route.hpp>
 
 namespace boost { 
 namespace archive {
 
 class polymorphic_binary_iarchive : 
-    public detail::polymorphic_iarchive_dispatch<naked_binary_iarchive>
+    public detail::polymorphic_iarchive_route<naked_binary_iarchive>
 {
 public:
     polymorphic_binary_iarchive(std::istream & is, unsigned int flags = 0) :
-        detail::polymorphic_iarchive_dispatch<naked_binary_iarchive>(is, flags)
+        detail::polymorphic_iarchive_route<naked_binary_iarchive>(is, flags)
     {}
     ~polymorphic_binary_iarchive(){}
 };
