@@ -37,8 +37,8 @@ template<class DeviceAbstraction, bool FileBased = true>
 class managed_open_or_create_impl
 {
    //Non-copyable
-   managed_open_or_create_impl(const managed_open_or_create_impl &);
-   managed_open_or_create_impl &operator=(const managed_open_or_create_impl &);
+   managed_open_or_create_impl(managed_open_or_create_impl &);
+   managed_open_or_create_impl &operator=(managed_open_or_create_impl &);
 
    enum
    {  
@@ -49,6 +49,7 @@ class managed_open_or_create_impl
    };
 
    public:
+   BOOST_INTERPROCESS_ENABLE_MOVE_EMULATION(managed_open_or_create_impl)
 
    static const std::size_t
       ManagedOpenOrCreateUserOffset = 
@@ -154,34 +155,16 @@ class managed_open_or_create_impl
          , construct_func);
    }
 
-
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   managed_open_or_create_impl(detail::moved_object<managed_open_or_create_impl> moved)
-   {  this->swap(moved.get());   }
-   #else
-   managed_open_or_create_impl(managed_open_or_create_impl &&moved)
+   managed_open_or_create_impl(BOOST_INTERPROCESS_RV_REF(managed_open_or_create_impl) moved)
    {  this->swap(moved);   }
-   #endif
 
-   //!Move assignment. If *this owns a memory mapped region, it will be
-   //!destroyed and it will take ownership of "other"'s memory mapped region.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   managed_open_or_create_impl &operator=(detail::moved_object<managed_open_or_create_impl> moved)
+   managed_open_or_create_impl &operator=(BOOST_INTERPROCESS_RV_REF(managed_open_or_create_impl) moved)
    {  
-      managed_open_or_create_impl tmp(moved);
+      managed_open_or_create_impl tmp(boost::interprocess::move(moved));
       this->swap(tmp);
       return *this;  
    }
 
-   #else
-   managed_open_or_create_impl &operator=(managed_open_or_create_impl &&moved)
-   {  
-      managed_open_or_create_impl tmp(detail::move_impl(moved));
-      this->swap(tmp);
-      return *this;  
-   }
-   #endif
-
    ~managed_open_or_create_impl()
    {}
 
@@ -215,40 +198,6 @@ class managed_open_or_create_impl
 
    private:
 
-   //These are templatized to allow explicit instantiations
-   template<bool dummy>
-   static void write_whole_device(DeviceAbstraction &, std::size_t, detail::false_)
-   {} //Empty
-
-   template<bool dummy>
-   static void write_whole_device(DeviceAbstraction &dev, std::size_t size, detail::true_)
-   {
-      file_handle_t hnd = detail::file_handle_from_mapping_handle(dev.get_mapping_handle());
-
-      if(size <= ManagedOpenOrCreateUserOffset){
-         throw interprocess_exception(error_info(system_error_code()));
-      }
-
-      size -= ManagedOpenOrCreateUserOffset;
-
-      if(!detail::set_file_pointer(hnd, ManagedOpenOrCreateUserOffset, file_begin)){
-         throw interprocess_exception(error_info(system_error_code()));
-      }
-
-      //We will write zeros in the file
-      for(std::size_t remaining = size, write_size = 0
-         ;remaining > 0
-         ;remaining -= write_size){
-         const std::size_t DataSize = 512;
-         static char data [DataSize];
-         write_size = DataSize < remaining ? DataSize : remaining;
-         if(!detail::write_file(hnd, data, write_size)){
-            error_info err = system_error_code();
-            throw interprocess_exception(err);
-         }
-      }
-   }
-
    //These are templatized to allow explicit instantiations
    template<bool dummy>
    static void truncate_device(DeviceAbstraction &, std::size_t, detail::false_)
@@ -359,7 +308,6 @@ class managed_open_or_create_impl
 
             if(previous == UninitializedSegment){
                try{
-                  write_whole_device<FileBased>(dev, size, file_like_t());
                   construct_func(static_cast<char*>(region.get_address()) + ManagedOpenOrCreateUserOffset, size - ManagedOpenOrCreateUserOffset, true);
                   //All ok, just move resources to the external mapped region
                   m_mapped_region.swap(region);
@@ -437,20 +385,6 @@ inline void swap(managed_open_or_create_impl<DeviceAbstraction> &x
 
 }  //namespace detail {
 
-
-///@cond
-
-//!Trait class to detect if a type is
-//!movable
-template<class DeviceAbstraction>
-
-struct is_movable<detail::managed_open_or_create_impl<DeviceAbstraction> >
-{
-   enum {  value = true };
-};
-
-///@endcond
-
 }  //namespace interprocess {
 }  //namespace boost {
 
