@@ -11,12 +11,17 @@
 
 #include <boost/proto/detail/prefix.hpp> // must be first include
 #include <boost/config.hpp>
+#include <boost/detail/workaround.hpp>
 #include <boost/get_pointer.hpp>
 #include <boost/preprocessor/cat.hpp>
 #include <boost/preprocessor/repetition/enum_params.hpp>
+#include <boost/preprocessor/repetition/enum_trailing_params.hpp>
 #include <boost/preprocessor/repetition/enum_binary_params.hpp>
 #include <boost/preprocessor/repetition/repeat_from_to.hpp>
+#include <boost/preprocessor/iteration/local.hpp>
 #include <boost/mpl/if.hpp>
+#include <boost/mpl/eval_if.hpp>
+#include <boost/mpl/identity.hpp>
 #include <boost/type_traits/is_class.hpp>
 #include <boost/type_traits/remove_cv.hpp>
 #include <boost/type_traits/remove_reference.hpp>
@@ -29,6 +34,7 @@
 #include <boost/utility/addressof.hpp>
 #include <boost/utility/result_of.hpp>
 #include <boost/utility/enable_if.hpp>
+#include <boost/proto/repeat.hpp>
 #include <boost/proto/detail/suffix.hpp> // must be last include
 
 // If we're generating doxygen documentation, hide all the nasty
@@ -108,7 +114,7 @@ namespace boost { namespace proto
                 friend any operator^(any, any);
                 friend any operator,(any, any);
                 friend any operator->*(any, any);
-                
+
                 friend any operator<<=(any, any);
                 friend any operator>>=(any, any);
                 friend any operator*=(any, any);
@@ -121,9 +127,9 @@ namespace boost { namespace proto
                 friend any operator^=(any, any);
             };
         }
-        
+
         using anyns::any;
-        
+
         ////////////////////////////////////////////////////////////////////////////////////////////
         template<typename T>
         struct as_mutable
@@ -157,7 +163,12 @@ namespace boost { namespace proto
           : T
         {
             using T::operator[];
-            any operator[](any) const volatile;
+
+            #if BOOST_WORKAROUND(BOOST_MSVC, BOOST_TESTED_AT(1500))
+            any operator[](any const volatile &) const volatile;
+            #else
+            any operator[](any const &) const volatile;
+            #endif
         };
 
         ////////////////////////////////////////////////////////////////////////////////////////////
@@ -220,7 +231,7 @@ namespace boost { namespace proto
         template<typename T>
         char (&check_reference(T const &))[2];
 
-        namespace has_get_pointer_
+        namespace has_get_pointerns
         {
             using boost::get_pointer;
             void *(&get_pointer(...))[2];
@@ -229,33 +240,138 @@ namespace boost { namespace proto
             template<typename T>
             struct has_get_pointer
             {
-                static T &t;
-                BOOST_STATIC_CONSTANT(bool, value = sizeof(void *) == sizeof(get_pointer(t)));
+                BOOST_STATIC_CONSTANT(bool, value = sizeof(void *) == sizeof(get_pointer(make<T &>())));
                 typedef mpl::bool_<value> type;
             };
         }
 
-        using has_get_pointer_::has_get_pointer;
-        
+        using has_get_pointerns::has_get_pointer;
+
         ////////////////////////////////////////////////////////////////////////////////////////////
-        namespace get_pointer_
+        template<typename T>
+        struct classtypeof;
+
+        template<typename T, typename U>
+        struct classtypeof<T U::*>
+        {
+            typedef U type;
+        };
+
+        #define BOOST_PP_LOCAL_MACRO(N)                                                             \
+        template<typename T, typename U BOOST_PP_ENUM_TRAILING_PARAMS(N, typename A)>               \
+        struct classtypeof<T (U::*)(BOOST_PP_ENUM_PARAMS(N, A))>                                    \
+        {                                                                                           \
+            typedef U type;                                                                         \
+        };                                                                                          \
+        template<typename T, typename U BOOST_PP_ENUM_TRAILING_PARAMS(N, typename A)>               \
+        struct classtypeof<T (U::*)(BOOST_PP_ENUM_PARAMS(N, A)) const>                              \
+        {                                                                                           \
+            typedef U type;                                                                         \
+        };                                                                                          \
+        /**/
+        #define BOOST_PP_LOCAL_LIMITS (0, BOOST_PROTO_MAX_ARITY)
+        #include BOOST_PP_LOCAL_ITERATE()
+
+        ////////////////////////////////////////////////////////////////////////////////////////////
+        template<typename T>
+        T &lvalue(T &t)
+        {
+            return t;
+        }
+
+        template<typename T>
+        T const &lvalue(T const &t)
+        {
+            return t;
+        }
+
+        ////////////////////////////////////////////////////////////////////////////////////////////
+        template<typename U, typename V, typename T>
+        U *proto_get_pointer(T &t, V *, U *)
+        {
+            return boost::addressof(t);
+        }
+
+        template<typename U, typename V, typename T>
+        U const *proto_get_pointer(T &t, V *, U const *)
+        {
+            return boost::addressof(t);
+        }
+
+        template<typename U, typename V, typename T>
+        V *proto_get_pointer(T &t, V *, ...)
+        {
+            return get_pointer(t);
+        }
+
+        ////////////////////////////////////////////////////////////////////////////////////////////
+        #define BOOST_PROTO_USE_GET_POINTER()                                                       \
+        using namespace boost::proto::detail::get_pointerns                                         \
+        /**/
+
+        #define BOOST_PROTO_GET_POINTER(Type, Obj)                                                  \
+        boost::proto::detail::proto_get_pointer<Type>(                                              \
+            boost::proto::detail::lvalue(Obj)                                                       \
+          , (true ? 0 : get_pointer(Obj))                                                           \
+          , (true ? 0 : boost::addressof(boost::proto::detail::lvalue(Obj)))                        \
+        )                                                                                           \
+        /**/
+
+        ////////////////////////////////////////////////////////////////////////////////////////////
+        namespace get_pointerns
         {
             using boost::get_pointer;
 
             template<typename T>
-            typename disable_if<has_get_pointer<T>, T *>::type
+            typename disable_if_c<has_get_pointer<T>::value, T *>::type
             get_pointer(T &t)
             {
                 return boost::addressof(t);
             }
 
             template<typename T>
-            typename disable_if<has_get_pointer<T>, T const *>::type
+            typename disable_if_c<has_get_pointer<T>::value, T const *>::type
             get_pointer(T const &t)
             {
                 return boost::addressof(t);
             }
-            
+
+            char test_ptr_to_const(void *);
+            char (&test_ptr_to_const(void const *))[2];
+
+            template<typename U> char test_V_is_a_U(U *);
+            template<typename U> char test_V_is_a_U(U const *);
+            template<typename U> char (&test_V_is_a_U(...))[2];
+
+            ////////////////////////////////////////////////////////////////////////////////////////////
+            // result_of_ is a wrapper around boost::result_of that also handles "invocations" of
+            // member object pointers.
+            template<typename T, typename Void = void>
+            struct result_of_
+              : boost::result_of<T>
+            {};
+
+            template<typename T, typename U, typename V>
+            struct result_of_<T U::*(V), typename enable_if_c<is_member_object_pointer<T U::*>::value>::type>
+            {
+                BOOST_STATIC_CONSTANT(bool, is_V_a_smart_ptr = 2 == sizeof(test_V_is_a_U<U>(&lvalue(make<V>()))));
+                BOOST_STATIC_CONSTANT(bool, is_ptr_to_const = 2 == sizeof(test_ptr_to_const(BOOST_PROTO_GET_POINTER(U, make<V>()))));
+
+                // If V is not a U, then it is a (smart) pointer and we can always return an lvalue.
+                // Otherwise, we can only return an lvalue if we are given one.
+                typedef
+                    typename mpl::eval_if_c<
+                        (is_V_a_smart_ptr || is_reference<V>::value)
+                      , mpl::eval_if_c<
+                            is_ptr_to_const
+                          , add_reference<typename add_const<T>::type>
+                          , add_reference<T>
+                        >
+                      , mpl::identity<T>
+                    >::type
+                type;
+            };
+
             ////////////////////////////////////////////////////////////////////////////////////////////
             template<
                 typename T
@@ -284,22 +400,31 @@ namespace boost { namespace proto
             template<typename T, typename U>
             struct mem_ptr_fun<T, U, true>
             {
+                typedef
+                    typename classtypeof<
+                        typename remove_const<
+                            typename remove_reference<U>::type
+                        >::type
+                    >::type
+                V;
+
                 BOOST_PROTO_DECLTYPE_(
-                    get_pointer(proto::detail::make_mutable<T>()) ->* proto::detail::make<U>()
+                    BOOST_PROTO_GET_POINTER(V, proto::detail::make_mutable<T>()) ->* proto::detail::make<U>()
                   , result_type
                 )
 
                 result_type operator()(
                     typename add_reference<typename add_const<T>::type>::type t
-                  , typename add_reference<typename add_const<U>::type>::type u
+                  , U u
                 ) const
                 {
-                    return get_pointer(t) ->* u;
+                    return BOOST_PROTO_GET_POINTER(V, t) ->* u;
                 }
             };
         }
 
-        using get_pointer_::mem_ptr_fun;
+        using get_pointerns::result_of_;
+        using get_pointerns::mem_ptr_fun;
 
         ////////////////////////////////////////////////////////////////////////////////////////////
         template<typename A0, typename A1>
@@ -327,51 +452,7 @@ namespace boost { namespace proto
         };
 
         ////////////////////////////////////////////////////////////////////////////////////////////
-        template<typename T>
-        struct result_of_member;
-        
-        template<typename T, typename U>
-        struct result_of_member<T U::*>
-        {
-            typedef T type;
-        };
-
-        template<typename T, typename Void = void>
-        struct result_of_
-          : boost::result_of<T>
-        {};
-        
-        template<typename T, typename U>
-        struct result_of_<T(U), typename enable_if<is_member_object_pointer<T> >::type>
-        {
-            typedef
-                typename result_of_member<T>::type
-            type;
-        };
-
-        template<typename T, typename U>
-        struct result_of_<T(U &), typename enable_if<is_member_object_pointer<T> >::type>
-        {
-            typedef
-                typename add_reference<
-                    typename result_of_member<T>::type
-                >::type
-            type;
-        };
-
-        template<typename T, typename U>
-        struct result_of_<T(U const &), typename enable_if<is_member_object_pointer<T> >::type>
-        {
-            typedef
-                typename add_reference<
-                    typename add_const<
-                        typename result_of_member<T>::type
-                    >::type
-                >::type
-            type;
-        };
-
-        ////////////////////////////////////////////////////////////////////////////////////////////
+        // normalize a function type for use with boost::result_of
         template<typename T, typename U = T>
         struct result_of_fixup
           : mpl::if_c<is_function<T>::value, T *, U>
@@ -418,29 +499,31 @@ namespace boost { namespace proto
         struct memfun
         {
             typedef typename remove_const<typename remove_reference<PMF>::type>::type pmf_type;
+            typedef typename classtypeof<pmf_type>::type V;
             typedef typename boost::result_of<pmf_type(T)>::type result_type;
 
-            memfun(T t, PMF pmf)
+            memfun(T t, PMF p)
               : obj(t)
-              , pmf(pmf)
+              , pmf(p)
             {}
 
             result_type operator()() const
             {
-                using namespace get_pointer_;
-                return (get_pointer(obj) ->* pmf)();
+                BOOST_PROTO_USE_GET_POINTER();
+                return (BOOST_PROTO_GET_POINTER(V, obj) ->* pmf)();
             }
 
-            #define M0(Z, N, DATA)                                                                  \
-            template<BOOST_PP_ENUM_PARAMS_Z(Z, N, typename A)>                                      \
-            result_type operator()(BOOST_PP_ENUM_BINARY_PARAMS_Z(Z, N, A, const &a)) const          \
+            #define BOOST_PROTO_LOCAL_MACRO(N, typename_A, A_const_ref, A_const_ref_a, a)           \
+            template<typename_A(N)>                                                                 \
+            result_type operator()(A_const_ref_a(N)) const                                          \
             {                                                                                       \
-                using namespace get_pointer_;                                                       \
-                return (get_pointer(obj) ->* pmf)(BOOST_PP_ENUM_PARAMS_Z(Z, N, a));                  \
+                BOOST_PROTO_USE_GET_POINTER();                                                      \
+                return (BOOST_PROTO_GET_POINTER(V, obj) ->* pmf)(a(N));                             \
             }                                                                                       \
             /**/
-            BOOST_PP_REPEAT_FROM_TO(1, BOOST_PROTO_MAX_ARITY, M0, ~)
-            #undef M0
+            #define BOOST_PROTO_LOCAL_a BOOST_PROTO_a
+            #define BOOST_PROTO_LOCAL_LIMITS (1, BOOST_PROTO_MAX_ARITY)
+            #include BOOST_PROTO_LOCAL_ITERATE()
 
         private:
             T obj;
