@@ -123,12 +123,12 @@ namespace boost { namespace proto
 
     /// INTERNAL ONLY
     ///
-    #define BOOST_PROTO_DEFINE_FUN_OP_CONST(Z, N, DATA)                                         \
+    #define BOOST_PROTO_DEFINE_FUN_OP_CONST(Z, N, DATA)                                             \
         BOOST_PROTO_DEFINE_FUN_OP_IMPL_(Z, N, DATA, 1)
 
     /// INTERNAL ONLY
     ///
-    #define BOOST_PROTO_DEFINE_FUN_OP_NON_CONST(Z, N, DATA)                                     \
+    #define BOOST_PROTO_DEFINE_FUN_OP_NON_CONST(Z, N, DATA)                                         \
         BOOST_PROTO_DEFINE_FUN_OP_IMPL_(Z, N, DATA, 0)
 
     /// INTERNAL ONLY
@@ -144,9 +144,6 @@ namespace boost { namespace proto
         typedef                                                                                     \
             typename proto_base_expr::BOOST_PP_CAT(proto_child, N)                                  \
         BOOST_PP_CAT(proto_child, N);                                                               \
-        typedef                                                                                     \
-            typename proto_base_expr::BOOST_PP_CAT(proto_child_ref, N)                              \
-        BOOST_PP_CAT(proto_child_ref, N);                                                           \
         /**/
 
     #define BOOST_PROTO_BASIC_EXTENDS_(Expr, Derived, Domain)                                       \
@@ -159,7 +156,8 @@ namespace boost { namespace proto
         typedef typename proto_base_expr::proto_args proto_args;                                    \
         typedef typename proto_base_expr::proto_arity proto_arity;                                  \
         typedef typename proto_base_expr::address_of_hack_type_ proto_address_of_hack_type_;        \
-        typedef void proto_is_expr_;                                                                \
+        typedef void proto_is_expr_; /**< INTERNAL ONLY */                                          \
+        BOOST_STATIC_CONSTANT(long, proto_arity_c = proto_base_expr::proto_arity_c);                \
         BOOST_PROTO_FUSION_DEFINE_TAG(boost::proto::tag::proto_expr)                                \
         BOOST_PP_REPEAT(BOOST_PROTO_MAX_ARITY, BOOST_PROTO_EXTENDS_CHILD, ~)                        \
                                                                                                     \
@@ -188,7 +186,7 @@ namespace boost { namespace proto
     #define BOOST_PROTO_BASIC_EXTENDS(Expr, Derived, Domain)                                        \
         BOOST_PROTO_BASIC_EXTENDS_(Expr, Derived, Domain)                                           \
         typedef void proto_is_aggregate_;                                                           \
-        /**/
+        /**< INTERNAL ONLY */
 
         /// INTERNAL ONLY
         ///
@@ -202,6 +200,7 @@ namespace boost { namespace proto
                         proto_derived_expr BOOST_PROTO_CONST ## Const &                             \
                       , typename boost::proto::result_of::as_child<A, proto_domain>::type           \
                     >                                                                               \
+                  , 2                                                                               \
                 >                                                                                   \
             )                                                                                       \
         >::type const                                                                               \
@@ -213,6 +212,7 @@ namespace boost { namespace proto
                     proto_derived_expr BOOST_PROTO_CONST ## Const &                                 \
                   , typename boost::proto::result_of::as_child<A, proto_domain>::type               \
                 >                                                                                   \
+              , 2                                                                                   \
             > that_type;                                                                            \
             that_type that = {                                                                      \
                 *static_cast<proto_derived_expr BOOST_PROTO_CONST ## Const *>(this)                 \
@@ -230,6 +230,7 @@ namespace boost { namespace proto
                         proto_derived_expr BOOST_PROTO_CONST ## Const &                             \
                       , typename boost::proto::result_of::as_child<A const, proto_domain>::type     \
                     >                                                                               \
+                  , 2                                                                               \
                 >                                                                                   \
             )                                                                                       \
         >::type const                                                                               \
@@ -241,6 +242,7 @@ namespace boost { namespace proto
                     proto_derived_expr BOOST_PROTO_CONST ## Const &                                 \
                   , typename boost::proto::result_of::as_child<A const, proto_domain>::type         \
                 >                                                                                   \
+              , 2                                                                                   \
             > that_type;                                                                            \
             that_type that = {                                                                      \
                 *static_cast<proto_derived_expr BOOST_PROTO_CONST ## Const *>(this)                 \
@@ -273,6 +275,7 @@ namespace boost { namespace proto
                         proto_derived_expr BOOST_PROTO_CONST ## Const &                             \
                       , typename boost::proto::result_of::as_child<A, proto_domain>::type           \
                     >                                                                               \
+                  , 2                                                                               \
                 >                                                                                   \
             )                                                                                       \
         >::type const                                                                               \
@@ -284,6 +287,7 @@ namespace boost { namespace proto
                     proto_derived_expr BOOST_PROTO_CONST ## Const &                                 \
                   , typename boost::proto::result_of::as_child<A, proto_domain>::type               \
                 >                                                                                   \
+              , 2                                                                                   \
             > that_type;                                                                            \
             that_type that = {                                                                      \
                 *static_cast<proto_derived_expr BOOST_PROTO_CONST ## Const *>(this)                 \
@@ -301,6 +305,7 @@ namespace boost { namespace proto
                         proto_derived_expr BOOST_PROTO_CONST ## Const &                             \
                       , typename boost::proto::result_of::as_child<A const, proto_domain>::type     \
                     >                                                                               \
+                  , 2                                                                               \
                 >                                                                                   \
             )                                                                                       \
         >::type const                                                                               \
@@ -312,6 +317,7 @@ namespace boost { namespace proto
                     proto_derived_expr BOOST_PROTO_CONST ## Const &                                 \
                   , typename boost::proto::result_of::as_child<A const, proto_domain>::type         \
                 >                                                                                   \
+              , 2                                                                                   \
             > that_type;                                                                            \
             that_type that = {                                                                      \
                 *static_cast<proto_derived_expr BOOST_PROTO_CONST ## Const *>(this)                 \
@@ -347,7 +353,7 @@ namespace boost { namespace proto
                           , proto_domain                                                            \
                         >::type                                                                     \
                     )                                                                               \
-                >::type                                                                             \
+                >::type const                                                                       \
             type;                                                                                   \
         };                                                                                          \
         /**/
@@ -442,8 +448,8 @@ namespace boost { namespace proto
     template<
         typename Expr
       , typename Derived
-      , typename Domain     BOOST_PROTO_WHEN_BUILDING_DOCS(= default_domain)
-      , typename Tag        BOOST_PROTO_WHEN_BUILDING_DOCS(= typename Expr::proto_tag)
+      , typename Domain     BOOST_PROTO_WHEN_BUILDING_DOCS(= proto::default_domain)
+      , long Arity          BOOST_PROTO_WHEN_BUILDING_DOCS(= Expr::proto_arity_c)
     >
     struct extends
     {
@@ -488,7 +494,7 @@ namespace boost { namespace proto
     /// \brief extends\<\> class template for adding behaviors to a Proto expression template
     ///
     template<typename Expr, typename Derived, typename Domain>
-    struct extends<Expr, Derived, Domain, tag::terminal>
+    struct extends<Expr, Derived, Domain, 0>
     {
         extends()
           : proto_expr_()
@@ -536,7 +542,7 @@ namespace boost { namespace proto
     struct virtual_member
     {
         typedef
-            expr<tag::member, list2<This &, expr<tag::terminal, term<Fun> > const &> >
+            expr<tag::member, list2<This &, expr<tag::terminal, term<Fun> > const &>, 2>
         proto_base_expr;
         typedef Domain proto_domain;
         typedef virtual_member<This, Fun, Domain> proto_derived_expr;
@@ -544,10 +550,11 @@ namespace boost { namespace proto
         typedef typename proto_base_expr::proto_args proto_args;
         typedef typename proto_base_expr::proto_arity proto_arity;
         typedef typename proto_base_expr::address_of_hack_type_ proto_address_of_hack_type_;
-        typedef void proto_is_expr_;
+        typedef void proto_is_expr_; /**< INTERNAL ONLY */
+        BOOST_STATIC_CONSTANT(long, proto_arity_c = proto_base_expr::proto_arity_c);
         BOOST_PROTO_FUSION_DEFINE_TAG(boost::proto::tag::proto_expr)
         BOOST_PP_REPEAT(BOOST_PROTO_MAX_ARITY, BOOST_PROTO_EXTENDS_CHILD, ~)
-        typedef void proto_is_aggregate_;
+        typedef void proto_is_aggregate_; /**< INTERNAL ONLY */
 
         BOOST_PROTO_EXTENDS_ASSIGN()
         BOOST_PROTO_EXTENDS_SUBSCRIPT()
@@ -566,7 +573,7 @@ namespace boost { namespace proto
 
         proto_child1 child1() const
         {
-            static expr<tag::terminal, term<Fun> > const that = {Fun()};
+            static expr<tag::terminal, term<Fun>, 0> const that = {Fun()};
             return that;
         }
     };
