@@ -27,9 +27,13 @@
 #include <boost/serialization/split_free.hpp>
 #include <boost/serialization/array.hpp>
 #include <boost/serialization/detail/get_data.hpp>
-#include <boost/detail/has_default_constructor.hpp>
 #include <boost/mpl/bool.hpp>
 
+// default is being compatible with version 1.34.1 files, not 1.35 files
+#ifndef BOOST_SERIALIZATION_VECTOR_VERSION
+#define BOOST_SERIALIZATION_VECTOR_VERSION 4
+#endif
+
 namespace boost { 
 namespace serialization {
 
@@ -78,8 +82,10 @@ inline void save(
 ){
     const collection_size_type count(t.size());
     ar << BOOST_SERIALIZATION_NVP(count);
+    const unsigned int item_version = version<U>::value;
+    ar << BOOST_SERIALIZATION_NVP(item_version);
     if (!t.empty())
-      ar << make_array(detail::get_data(t),t.size());
+        ar << make_array(detail::get_data(t),t.size());
 }
 
 template<class Archive, class U, class Allocator>
@@ -92,8 +98,11 @@ inline void load(
     collection_size_type count(t.size());
     ar >> BOOST_SERIALIZATION_NVP(count);
     t.resize(count);
+    unsigned int item_version=0;
+    if(BOOST_SERIALIZATION_VECTOR_VERSION < ar.get_library_version())
+        ar >> BOOST_SERIALIZATION_NVP(item_version);
     if (!t.empty())
-      ar >> make_array(detail::get_data(t),t.size());
+        ar >> make_array(detail::get_data(t),t.size());
   }
 
 // dispatch to either default or optimized versions
@@ -104,7 +113,11 @@ inline void save(
     const std::vector<U, Allocator> &t,
     const unsigned int file_version
 ){
-    save(ar,t,file_version, boost::detail::has_default_constructor<U>());
+    typedef BOOST_DEDUCED_TYPENAME 
+    boost::serialization::use_array_optimization<Archive>::template apply<
+        BOOST_DEDUCED_TYPENAME remove_const<U>::type 
+    >::type use_optimized;
+    save(ar,t,file_version, use_optimized());
 }
 
 template<class Archive, class U, class Allocator>
@@ -113,7 +126,11 @@ inline void load(
     std::vector<U, Allocator> &t,
     const unsigned int file_version
 ){
-    load(ar,t,file_version, boost::detail::has_default_constructor<U>());
+    typedef BOOST_DEDUCED_TYPENAME 
+    boost::serialization::use_array_optimization<Archive>::template apply<
+        BOOST_DEDUCED_TYPENAME remove_const<U>::type 
+    >::type use_optimized;
+    load(ar,t,file_version, use_optimized());
 }
 
 // split non-intrusive serialization function member into separate
