@@ -5,7 +5,7 @@
     
     http://www.boost.org/
 
-    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2009 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -40,6 +40,7 @@
 #include <boost/wave/util/symbol_table.hpp>
 #include <boost/wave/util/cpp_macromap_utils.hpp>
 #include <boost/wave/util/cpp_macromap_predef.hpp>
+#include <boost/wave/util/filesystem_compatibility.hpp>
 #include <boost/wave/grammars/cpp_defined_grammar_gen.hpp>
 
 #include <boost/wave/wave_version.hpp>
@@ -861,12 +862,13 @@ macromap<ContextT>::expand_whole_tokensequence(ContainerT &expanded,
         gen_type;
     typedef typename gen_type::return_type iterator_type;
 
-iterator_type first_it = gen_type::generate(first);
-iterator_type last_it = gen_type::generate(last);
+    ContainerT empty;
+    iterator_type first_it = gen_type::generate(empty, first);
+    iterator_type last_it = gen_type::generate(last);
 
-on_exit::assign<IteratorT, iterator_type> on_exit(first, first_it);
-ContainerT pending_queue;
-bool seen_newline;
+    on_exit::assign<IteratorT, iterator_type> on_exit(first, first_it);
+    ContainerT pending_queue;
+    bool seen_newline;
     
     while (!pending_queue.empty() || first_it != last_it) {
         expanded.push_back(
@@ -1414,10 +1416,10 @@ string_type const &value = curr_token.get_value();
         namespace fs = boost::filesystem;
         
     std::string file("\"");
-    fs::path filename(main_pos.get_file().c_str(), fs::native);
+    fs::path filename(wave::util::create_path(main_pos.get_file().c_str()));
     
         using boost::wave::util::impl::escape_lit;
-        file += escape_lit(filename.native_file_string()) + "\"";
+        file += escape_lit(wave::util::native_file_string(filename)) + "\"";
         expanded.push_back(token_type(T_STRINGLIT, file.c_str(), 
             curr_token.get_position()));
         return true;
@@ -1846,21 +1848,21 @@ position_type pos("<built-in>");
 // predefine the __BASE_FILE__ macro which contains the main file name 
     namespace fs = boost::filesystem; 
     if (string_type(fname) != "<Unknown>") {
-    fs::path filename(fname, fs::native);
+    fs::path filename(create_path(fname));
     
         using boost::wave::util::impl::escape_lit;
         predefine_macro(current_scope, "__BASE_FILE__",
             token_type(T_STRINGLIT, string_type("\"") + 
-                escape_lit(filename.native_file_string()).c_str() + "\"", pos));
+                escape_lit(native_file_string(filename)).c_str() + "\"", pos));
         base_name = fname;
     }
     else if (!base_name.empty()) {
-    fs::path filename(base_name.c_str(), fs::native);
+    fs::path filename(create_path(base_name.c_str()));
     
         using boost::wave::util::impl::escape_lit;
         predefine_macro(current_scope, "__BASE_FILE__",
             token_type(T_STRINGLIT, string_type("\"") + 
-                escape_lit(filename.native_file_string()).c_str() + "\"", pos));
+                escape_lit(native_file_string(filename)).c_str() + "\"", pos));
     }
     
 // now add the dynamic macros
