@@ -1,7 +1,7 @@
 ///////////////////////////////////////////////////////////////////////////////
 // sub_match_impl.hpp
 //
-//  Copyright 2007 Eric Niebler. Distributed under the Boost
+//  Copyright 2008 Eric Niebler. Distributed under the Boost
 //  Software License, Version 1.0. (See accompanying file
 //  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -22,6 +22,13 @@ namespace boost { namespace xpressive { namespace detail
 // of them can be made more efficient if they are. Or maybe all they
 // need is trivial constructor/destructor. (???)
 
+///////////////////////////////////////////////////////////////////////////////
+// sub_match_impl_default
+//
+struct sub_match_impl_default
+{
+};
+
 ///////////////////////////////////////////////////////////////////////////////
 // sub_match_impl
 //
@@ -40,6 +47,14 @@ struct sub_match_impl
       , zero_width_(false)
     {
     }
+
+    sub_match_impl &operator =(sub_match_impl_default const &)
+    {
+        this->matched = false;
+        this->repeat_count_ = 0;
+        this->zero_width_ = false;
+        return *this;
+    }
 };
 
 }}} // namespace boost::xpressive::detail
