@@ -49,7 +49,7 @@ void test()
 
 //------------------------------------------------------------------------------
 
-test_suite* init_unit_test_suite(int argc, char* argv[])
+test_suite* init_unit_test_suite(int, char*[])
 {
   test_suite* test = BOOST_TEST_SUITE("ip/host_name");
   test->add(BOOST_TEST_CASE(&ip_host_name_compile::test));
