@@ -11,7 +11,7 @@
 
 //  See http://www.boost.org/libs/config for most recent version.
 //
-//  Revision $Id: config_info.cpp 49212 2008-10-09 16:08:03Z johnmaddock $
+//  Revision $Id: config_info.cpp 53445 2009-05-30 11:22:59Z bemandawes $
 //
 
 #include <boost/config.hpp>
@@ -154,9 +154,11 @@ void print_compiler_macros()
   // GNUC options:
   PRINT_MACRO(__GNUC__);
   PRINT_MACRO(__GNUC_MINOR__);
+  PRINT_MACRO(__GNUC_PATCHLEVEL__);
   PRINT_MACRO(__STDC_VERSION__);
   PRINT_MACRO(__GNUG__);
   PRINT_MACRO(__STRICT_ANSI__);
+  PRINT_MACRO(__GXX_EXPERIMENTAL_CXX0X__);
   PRINT_MACRO(__VERSION__);
   PRINT_MACRO(__OPTIMIZE__);
   PRINT_MACRO(__CHAR_UNSIGNED__);
@@ -952,30 +954,65 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_HAS_WINTHREADS);
    PRINT_MACRO(BOOST_MSVC6_MEMBER_TEMPLATES);
    PRINT_MACRO(BOOST_MSVC_STD_ITERATOR);
+   PRINT_MACRO(BOOST_NO_0X_HDR_ARRAY);
+   PRINT_MACRO(BOOST_NO_0X_HDR_CHRONO);
+   PRINT_MACRO(BOOST_NO_0X_HDR_CODECVT);
+   PRINT_MACRO(BOOST_NO_0X_HDR_CONCEPTS);
+   PRINT_MACRO(BOOST_NO_0X_HDR_CONDITION_VARIABLE);
+   PRINT_MACRO(BOOST_NO_0X_HDR_CONTAINER_CONCEPTS);
+   PRINT_MACRO(BOOST_NO_0X_HDR_FORWARD_LIST);
+   PRINT_MACRO(BOOST_NO_0X_HDR_FUTURE);
+   PRINT_MACRO(BOOST_NO_0X_HDR_INITIALIZER_LIST);
+   PRINT_MACRO(BOOST_NO_0X_HDR_ITERATOR_CONCEPTS);
+   PRINT_MACRO(BOOST_NO_0X_HDR_MEMORY_CONCEPTS);
+   PRINT_MACRO(BOOST_NO_0X_HDR_MUTEX);
+   PRINT_MACRO(BOOST_NO_0X_HDR_RANDOM);
+   PRINT_MACRO(BOOST_NO_0X_HDR_RATIO);
+   PRINT_MACRO(BOOST_NO_0X_HDR_REGEX);
+   PRINT_MACRO(BOOST_NO_0X_HDR_SYSTEM_ERROR);
+   PRINT_MACRO(BOOST_NO_0X_HDR_THREAD);
+   PRINT_MACRO(BOOST_NO_0X_HDR_TUPLE);
+   PRINT_MACRO(BOOST_NO_0X_HDR_TYPE_TRAITS);
+   PRINT_MACRO(BOOST_NO_0X_HDR_UNORDERED_MAP);
+   PRINT_MACRO(BOOST_NO_0X_HDR_UNORDERED_SET);
    PRINT_MACRO(BOOST_NO_ADL_BARRIER);
    PRINT_MACRO(BOOST_NO_ARGUMENT_DEPENDENT_LOOKUP);
    PRINT_MACRO(BOOST_NO_ARRAY_TYPE_SPECIALIZATIONS);
+   PRINT_MACRO(BOOST_NO_AUTO_DECLARATIONS);
+   PRINT_MACRO(BOOST_NO_AUTO_MULTIDECLARATIONS);
    PRINT_MACRO(BOOST_NO_AUTO_PTR);
+   PRINT_MACRO(BOOST_NO_CHAR16_T);
+   PRINT_MACRO(BOOST_NO_CHAR32_T);
+   PRINT_MACRO(BOOST_NO_CONCEPTS);
+   PRINT_MACRO(BOOST_NO_CONSTEXPR);
    PRINT_MACRO(BOOST_NO_CTYPE_FUNCTIONS);
    PRINT_MACRO(BOOST_NO_CV_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_CV_VOID_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_CWCHAR);
    PRINT_MACRO(BOOST_NO_CWCTYPE);
+   PRINT_MACRO(BOOST_NO_DECLTYPE);
+   PRINT_MACRO(BOOST_NO_DEFAULTED_FUNCTIONS);
+   PRINT_MACRO(BOOST_NO_DELETED_FUNCTIONS);
    PRINT_MACRO(BOOST_NO_DEPENDENT_NESTED_DERIVATIONS);
    PRINT_MACRO(BOOST_NO_DEPENDENT_TYPES_IN_TEMPLATE_VALUE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_EXCEPTIONS);
    PRINT_MACRO(BOOST_NO_EXCEPTION_STD_NAMESPACE);
+   PRINT_MACRO(BOOST_NO_EXPLICIT_CONVERSION_OPERATORS);
    PRINT_MACRO(BOOST_NO_EXPLICIT_FUNCTION_TEMPLATE_ARGUMENTS);
+   PRINT_MACRO(BOOST_NO_EXTERN_TEMPLATE);
    PRINT_MACRO(BOOST_NO_FUNCTION_TEMPLATE_ORDERING);
    PRINT_MACRO(BOOST_NO_FUNCTION_TYPE_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_INCLASS_MEMBER_INITIALIZATION);
+   PRINT_MACRO(BOOST_NO_INITIALIZER_LISTS);
    PRINT_MACRO(BOOST_NO_INTEGRAL_INT64_T);
    PRINT_MACRO(BOOST_NO_INTRINSIC_WCHAR_T);
    PRINT_MACRO(BOOST_NO_IOSFWD);
    PRINT_MACRO(BOOST_NO_IOSTREAM);
    PRINT_MACRO(BOOST_NO_IS_ABSTRACT);
+   PRINT_MACRO(BOOST_NO_LAMBDAS);
    PRINT_MACRO(BOOST_NO_LIMITS);
    PRINT_MACRO(BOOST_NO_LIMITS_COMPILE_TIME_CONSTANTS);
+   PRINT_MACRO(BOOST_NO_LONG_LONG);
    PRINT_MACRO(BOOST_NO_LONG_LONG_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_MEMBER_FUNCTION_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATES);
@@ -983,13 +1020,18 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_MEMBER_TEMPLATE_KEYWORD);
    PRINT_MACRO(BOOST_NO_MS_INT64_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_NESTED_FRIENDSHIP);
+   PRINT_MACRO(BOOST_NO_NULLPTR);
    PRINT_MACRO(BOOST_NO_OPERATORS_IN_NAMESPACE);
    PRINT_MACRO(BOOST_NO_PARTIAL_SPECIALIZATION_IMPLICIT_DEFAULT_ARGS);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_CONST);
    PRINT_MACRO(BOOST_NO_POINTER_TO_MEMBER_TEMPLATE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_PRIVATE_IN_AGGREGATE);
+   PRINT_MACRO(BOOST_NO_RAW_LITERALS);
    PRINT_MACRO(BOOST_NO_RTTI);
+   PRINT_MACRO(BOOST_NO_RVALUE_REFERENCES);
+   PRINT_MACRO(BOOST_NO_SCOPED_ENUMS);
    PRINT_MACRO(BOOST_NO_SFINAE);
+   PRINT_MACRO(BOOST_NO_STATIC_ASSERT);
    PRINT_MACRO(BOOST_NO_STDC_NAMESPACE);
    PRINT_MACRO(BOOST_NO_STD_ALLOCATOR);
    PRINT_MACRO(BOOST_NO_STD_DISTANCE);
@@ -1000,6 +1042,7 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_STD_MIN_MAX);
    PRINT_MACRO(BOOST_NO_STD_OUTPUT_ITERATOR_ASSIGN);
    PRINT_MACRO(BOOST_NO_STD_TYPEINFO);
+   PRINT_MACRO(BOOST_NO_STD_UNORDERED);
    PRINT_MACRO(BOOST_NO_STD_USE_FACET);
    PRINT_MACRO(BOOST_NO_STD_WSTREAMBUF);
    PRINT_MACRO(BOOST_NO_STD_WSTRING);
@@ -1007,33 +1050,22 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_SWPRINTF);
    PRINT_MACRO(BOOST_NO_TEMPLATED_IOSTREAMS);
    PRINT_MACRO(BOOST_NO_TEMPLATED_ITERATOR_CONSTRUCTORS);
+   PRINT_MACRO(BOOST_NO_TEMPLATE_ALIASES);
    PRINT_MACRO(BOOST_NO_TEMPLATE_PARTIAL_SPECIALIZATION);
    PRINT_MACRO(BOOST_NO_TEMPLATE_TEMPLATES);
    PRINT_MACRO(BOOST_NO_TWO_PHASE_NAME_LOOKUP);
    PRINT_MACRO(BOOST_NO_TYPEID);
    PRINT_MACRO(BOOST_NO_TYPENAME_WITH_CTOR);
+   PRINT_MACRO(BOOST_NO_UNICODE_LITERALS);
    PRINT_MACRO(BOOST_NO_UNREACHABLE_RETURN_DETECTION);
    PRINT_MACRO(BOOST_NO_USING_DECLARATION_OVERLOADS_FROM_TYPENAME_BASE);
    PRINT_MACRO(BOOST_NO_USING_TEMPLATE);
+   PRINT_MACRO(BOOST_NO_VARIADIC_TEMPLATES);
    PRINT_MACRO(BOOST_NO_VOID_RETURNS);
 
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 
 
 
@@ -1069,14 +1101,3 @@ int main()
 
   return 0;
 }
-
-
-
-
-
-
-
-
-
-
-
