@@ -54,14 +54,7 @@ using boost::system::system_category;
 
 # if defined(BOOST_WINDOWS_API)
 #   include <windows.h>
-#   if defined(__BORLANDC__) || defined(__MWERKS__)
-#     if defined(__BORLANDC__)
-        using std::time_t;
-#     endif
-#     include <utime.h>
-#   else
-#     include <sys/utime.h>
-#   endif
+#   include <ctime>  // for time_t
 
 # else // BOOST_POSIX_API
 #   include <sys/types.h>
@@ -90,7 +83,8 @@ using boost::system::system_category;
 //
 // TODO: find out what macros indicate dirent::d_type present in more libraries
 # if defined(BOOST_WINDOWS_API) \
-  || defined(_DIRENT_HAVE_D_TYPE) // defined by GNU C library if d_type present
+  || (defined(_DIRENT_HAVE_D_TYPE) /* defined by GNU C library if d_type present */ \
+    && !(defined(__SUNPRO_CC) && !defined(__sun)))  // _DIRENT_HAVE_D_TYPE wrong for Sun compiler on Linux
 #   define BOOST_FILESYSTEM_STATUS_CACHE
 # endif
 
@@ -198,6 +192,7 @@ namespace
       if ((ec.value() == ERROR_FILE_NOT_FOUND)
         || (ec.value() == ERROR_PATH_NOT_FOUND)
         || (ec.value() == ERROR_INVALID_NAME) // "tools/jam/src/:sys:stat.h", "//foo"
+        || (ec.value() == ERROR_INVALID_DRIVE) // USB card reader with no card inserted
         || (ec.value() == ERROR_INVALID_PARAMETER) // ":sys:stat.h"
         || (ec.value() == ERROR_BAD_PATHNAME) // "//nosuch" on Win64
         || (ec.value() == ERROR_BAD_NETPATH)) // "//nosuch" on Win32
@@ -583,7 +578,7 @@ namespace
         // rmdir() or unlink() as indicated.
         // Same bug also reported for QNX, with the same fix.
         int err = ::unlink( p );
-        if ( err != EPERM )
+        if ( err == 0 || errno != EPERM )
           return err;
         return ::rmdir( p );
 #     else
@@ -704,9 +699,9 @@ namespace boost
       }
 
       BOOST_FILESYSTEM_DECL error_code
-      copy_file_api( const std::wstring & from, const std::wstring & to )
+      copy_file_api( const std::wstring & from, const std::wstring & to, bool fail_if_exists )
       {
-        return error_code( ::CopyFileW( from.c_str(), to.c_str(), /*fail_if_exists=*/true )
+        return error_code( ::CopyFileW( from.c_str(), to.c_str(), fail_if_exists )
           ? 0 : ::GetLastError(), system_category );
       }
 
@@ -884,9 +879,9 @@ namespace boost
       }
 
       BOOST_FILESYSTEM_DECL error_code
-      copy_file_api( const std::string & from, const std::string & to )
+      copy_file_api( const std::string & from, const std::string & to, bool fail_if_exists )
       {
-        return error_code( ::CopyFileA( from.c_str(), to.c_str(), /*fail_if_exists=*/true )
+        return error_code( ::CopyFileA( from.c_str(), to.c_str(), fail_if_exists )
           ? 0 : ::GetLastError(), system_category );
       }
 
@@ -1201,22 +1196,30 @@ namespace boost
 
       BOOST_FILESYSTEM_DECL error_code
       copy_file_api( const std::string & from_file_ph,
-        const std::string & to_file_ph )
+        const std::string & to_file_ph, bool fail_if_exists )
       {
         const std::size_t buf_sz = 32768;
         boost::scoped_array<char> buf( new char [buf_sz] );
-        int infile=0, outfile=0;  // init quiets compiler warning
-        struct stat from_stat;
+        int infile=-1, outfile=-1;  // -1 means not open
 
-        if ( ::stat( from_file_ph.c_str(), &from_stat ) != 0
-          || (infile = ::open( from_file_ph.c_str(),
-                              O_RDONLY )) < 0
-          || (outfile = ::open( to_file_ph.c_str(),
-                                O_WRONLY | O_CREAT | O_EXCL,
-                                from_stat.st_mode )) < 0 )
+        // bug fixed: code previously did a stat() on the from_file first, but that
+        // introduced a gratuitous race condition; the stat() is now done after the open()
+
+        if ( (infile = ::open( from_file_ph.c_str(), O_RDONLY )) < 0 )
+          { return error_code( errno, system_category ); }
+
+        struct stat from_stat;
+        if ( ::stat( from_file_ph.c_str(), &from_stat ) != 0 )
+          { return error_code( errno, system_category ); }
+
+        int oflag = O_CREAT | O_WRONLY;
+        if ( fail_if_exists ) oflag |= O_EXCL;
+        if (  (outfile = ::open( to_file_ph.c_str(), oflag, from_stat.st_mode )) < 0 )
         {
-          if ( infile >= 0 ) ::close( infile );
-          return error_code( errno, system_category );
+          int open_errno = errno;
+          BOOST_ASSERT( infile >= 0 );
+          ::close( infile );
+          return error_code( open_errno, system_category );
         }
 
         ssize_t sz, sz_read=1, sz_write;
