@@ -1,5 +1,5 @@
 
-// Copyright 2005-2007 Daniel James.
+// Copyright 2005-2009 Daniel James.
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -15,7 +15,7 @@
 #include <boost/detail/lightweight_test.hpp>
 
 #include <boost/preprocessor/cat.hpp>
-#include <boost/limits.hpp>
+#include <boost/functional/hash/detail/limits.hpp>
 #include <boost/mpl/assert.hpp>
 #include <boost/type_traits/is_base_and_derived.hpp>
 
@@ -31,7 +31,7 @@
 template <class T>
 void numeric_test(T*)
 {
-    typedef std::numeric_limits<T> limits;
+    typedef boost::hash_detail::limits<T> limits;
 
     compile_time_tests((T*) 0);
 
@@ -55,7 +55,7 @@ void numeric_test(T*)
 
     if (limits::is_integer)
     {
-        if(limits::is_signed || limits::digits <= std::numeric_limits<std::size_t>::digits)
+        if(limits::is_signed || limits::digits <= boost::hash_detail::limits<std::size_t>::digits)
             BOOST_TEST(HASH_NAMESPACE::hash_value(T(-5)) == (std::size_t)T(-5));
         BOOST_TEST(HASH_NAMESPACE::hash_value(T(0)) == (std::size_t)T(0u));
         BOOST_TEST(HASH_NAMESPACE::hash_value(T(10)) == (std::size_t)T(10u));
@@ -67,7 +67,7 @@ void numeric_test(T*)
 template <class T>
 void limits_test(T*)
 {
-    typedef std::numeric_limits<T> limits;
+    typedef boost::hash_detail::limits<T> limits;
 
     if(limits::is_specialized)
     {
@@ -98,7 +98,7 @@ void limits_test(T*)
 template <class T>
 void poor_quality_tests(T*)
 {
-    typedef std::numeric_limits<T> limits;
+    typedef boost::hash_detail::limits<T> limits;
 
     HASH_NAMESPACE::hash<T> x1;
     HASH_NAMESPACE::hash<T> x2;
@@ -150,13 +150,16 @@ int main()
     NUMERIC_TEST(unsigned long, ulong)
 
 #if defined(BOOST_HAS_LONG_LONG)
-    NUMERIC_TEST_NO_LIMITS(long long, hash_longlong)
-    NUMERIC_TEST_NO_LIMITS(unsigned long long, ulonglong)
+    NUMERIC_TEST_NO_LIMITS(boost::long_long_type, long_long)
+    NUMERIC_TEST_NO_LIMITS(boost::ulong_long_type, ulong_long)
 #endif
 
     NUMERIC_TEST(float, float)
     NUMERIC_TEST(double, double)
 
+    NUMERIC_TEST(std::size_t, size_t)
+    NUMERIC_TEST(std::ptrdiff_t, ptrdiff_t)
+
     bool_test();
 
     return boost::report_errors();
