@@ -51,6 +51,14 @@ void recursive_list_test()//Test for recursive types
 int main ()
 {
    recursive_list_test();
+   {
+      //Now test move semantics
+      list<recursive_list> original;
+      list<recursive_list> move_ctor(boost::interprocess::move(original));
+      list<recursive_list> move_assign;
+      move_assign = boost::interprocess::move(move_ctor);
+      move_assign.swap(original);
+   }
    if(test::list_test<managed_shared_memory, MyList, true>())
       return 1;
 
