@@ -1,7 +1,7 @@
 /////////////////////////////////////////////////////////////////////////////
 //
 // (C) Copyright Olaf Krzikalla 2004-2006.
-// (C) Copyright Ion Gaztanaga  2006-2007.
+// (C) Copyright Ion Gaztanaga  2006-2008.
 //
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
@@ -24,6 +24,30 @@
 
 using namespace boost::intrusive;
 
+struct my_tag;
+
+template<class VoidPointer>
+struct hooks
+{
+   typedef unordered_set_base_hook<void_pointer<VoidPointer> >    base_hook_type;
+   typedef unordered_set_base_hook
+      < link_mode<auto_unlink>
+      , void_pointer<VoidPointer>
+      , tag<my_tag>
+      , store_hash<true>
+      >                                                           auto_base_hook_type;
+
+   typedef unordered_set_member_hook
+      < void_pointer<VoidPointer>
+      , optimize_multikey<true>
+      >                                                           member_hook_type;
+   typedef unordered_set_member_hook
+      < link_mode<auto_unlink>, void_pointer<VoidPointer>
+      , store_hash<true>
+      , optimize_multikey<true>
+      >                                                           auto_member_hook_type;
+};
+
 static const std::size_t BucketSize = 8;
 
 template<class ValueTraits, bool CacheBegin, bool CompareHash, bool Incremental>
@@ -532,15 +556,15 @@ class test_main_template
    public:
    int operator()()
    {
-      typedef testvalue<VoidPointer, constant_time_size> value_type;
+      typedef testvalue<hooks<VoidPointer> , constant_time_size> value_type;
       static const int random_init[6] = { 3, 2, 4, 1, 5, 2 };
-      std::vector<testvalue<VoidPointer, constant_time_size> > data (6);
+      std::vector<testvalue<hooks<VoidPointer> , constant_time_size> > data (6);
       for (int i = 0; i < 6; ++i)
          data[i].value_ = random_init[i]; 
 
       test_unordered_set < typename detail::get_base_value_traits
                   < value_type
-                  , typename value_type::unordered_set_base_hook_t
+                  , typename hooks<VoidPointer>::base_hook_type
                   >::type
                 , true
                 , false
@@ -549,8 +573,8 @@ class test_main_template
       test_unordered_set < typename detail::get_member_value_traits
                   < value_type
                   , member_hook< value_type
-                               , typename value_type::unordered_set_member_hook_t
-                               , &value_type::unordered_set_node_
+                               , typename hooks<VoidPointer>::member_hook_type
+                               , &value_type::node_
                                >
                   >::type
                 , false
@@ -568,15 +592,15 @@ class test_main_template<VoidPointer, false, incremental>
    public:
    int operator()()
    {
-      typedef testvalue<VoidPointer, false> value_type;
+      typedef testvalue<hooks<VoidPointer> , false> value_type;
       static const int random_init[6] = { 3, 2, 4, 1, 5, 2 };
-      std::vector<testvalue<VoidPointer, false> > data (6);
+      std::vector<testvalue<hooks<VoidPointer> , false> > data (6);
       for (int i = 0; i < 6; ++i)
          data[i].value_ = random_init[i]; 
 
       test_unordered_set < typename detail::get_base_value_traits
                   < value_type
-                  , typename value_type::unordered_set_base_hook_t
+                  , typename hooks<VoidPointer>::base_hook_type
                   >::type
                 , true
                 , false
@@ -586,8 +610,8 @@ class test_main_template<VoidPointer, false, incremental>
       test_unordered_set < typename detail::get_member_value_traits
                   < value_type
                   , member_hook< value_type
-                               , typename value_type::unordered_set_member_hook_t
-                               , &value_type::unordered_set_node_
+                               , typename hooks<VoidPointer>::member_hook_type
+                               , &value_type::node_
                                >
                   >::type
                 , false
@@ -597,9 +621,9 @@ class test_main_template<VoidPointer, false, incremental>
 
       test_unordered_set < typename detail::get_base_value_traits
                   < value_type
-                  , typename value_type::unordered_set_auto_base_hook_t
+                  , typename hooks<VoidPointer>::auto_base_hook_type
                   >::type
-                , true
+                , false
                 , true
                 , incremental
                 >::test_all(data);
@@ -607,8 +631,8 @@ class test_main_template<VoidPointer, false, incremental>
       test_unordered_set < typename detail::get_member_value_traits
                   < value_type
                   , member_hook< value_type
-                               , typename value_type::unordered_set_auto_member_hook_t
-                               , &value_type::unordered_set_auto_node_
+                               , typename hooks<VoidPointer>::auto_member_hook_type
+                               , &value_type::auto_node_
                                >
                   >::type
                 , false
