@@ -6,6 +6,7 @@
 #include <boost/python/docstring_options.hpp>
 #include <boost/python/object/function_object.hpp>
 #include <boost/python/object/function_handle.hpp>
+#include <boost/python/object/function_doc_signature.hpp>
 #include <boost/python/errors.hpp>
 #include <boost/python/str.hpp>
 #include <boost/python/object_attributes.hpp>
@@ -17,6 +18,7 @@
 #include <boost/python/ssize_t.hpp>
 
 #include <boost/python/detail/signature.hpp>
+#include <boost/python/detail/none.hpp>
 #include <boost/mpl/vector/vector10.hpp>
 
 #include <boost/bind.hpp>
@@ -30,7 +32,12 @@
 
 namespace boost { namespace python {
   volatile bool docstring_options::show_user_defined_ = true;
-  volatile bool docstring_options::show_signatures_ = true;
+  volatile bool docstring_options::show_cpp_signatures_ = true;
+#ifndef BOOST_PYTHON_NO_PY_SIGNATURES
+  volatile bool docstring_options::show_py_signatures_ = true;
+#else
+  volatile bool docstring_options::show_py_signatures_ = false;
+#endif
 }}
 
 namespace boost { namespace python { namespace objects { 
@@ -159,7 +166,7 @@ PyObject* function::call(PyObject* args, PyObject* keywords) const
                     else
                     {
                         // build a new arg tuple, will adjust its size later
-                        assert(max_arity <= ssize_t_max);
+                        assert(max_arity <= static_cast<std::size_t>(ssize_t_max));
                         inner_args = handle<>(
                             PyTuple_New(static_cast<ssize_t>(max_arity)));
 
@@ -290,7 +297,7 @@ object function::signatures(bool show_return_type) const
 void function::argument_error(PyObject* args, PyObject* /*keywords*/) const
 {
     static handle<> exception(
-        PyErr_NewException("Boost.Python.ArgumentError", PyExc_TypeError, 0));
+        PyErr_NewException(const_cast<char*>("Boost.Python.ArgumentError"), PyExc_TypeError, 0));
 
     object message = "Python argument types in\n    %s.%s("
         % make_tuple(this->m_namespace, this->m_name);
@@ -411,6 +418,12 @@ void function::add_to_namespace(
     add_to_namespace(name_space, name_, attribute, 0);
 }
 
+namespace detail
+{
+    extern char py_signature_tag[];
+    extern char cpp_signature_tag[];
+}
+
 void function::add_to_namespace(
     object const& name_space, char const* name_, object const& attribute, char const* doc)
 {
@@ -427,7 +440,7 @@ void function::add_to_namespace(
         else if (PyType_Check(ns))
             dict = ((PyTypeObject*)ns)->tp_dict;
         else    
-            dict = PyObject_GetAttrString(ns, "__dict__");
+            dict = PyObject_GetAttrString(ns, const_cast<char*>("__dict__"));
 
         if (dict == 0)
             throw_error_already_set();
@@ -474,7 +487,7 @@ void function::add_to_namespace(
             new_func->m_name = name;
 
         handle<> name_space_name(
-            allow_null(::PyObject_GetAttrString(name_space.ptr(), "__name__")));
+            allow_null(::PyObject_GetAttrString(name_space.ptr(), const_cast<char*>("__name__"))));
         
         if (name_space_name)
             new_func->m_namespace = object(name_space_name);
@@ -487,6 +500,7 @@ void function::add_to_namespace(
         throw_error_already_set();
 
     object mutable_attribute(attribute);
+/*
     if (doc != 0 && docstring_options::show_user_defined_)
     {
         // Accumulate documentation
@@ -507,7 +521,8 @@ void function::add_to_namespace(
     {
         if (   PyObject_HasAttrString(mutable_attribute.ptr(), "__doc__")
             && mutable_attribute.attr("__doc__")) {
-            mutable_attribute.attr("__doc__") += "\n";
+            mutable_attribute.attr("__doc__") += (
+              mutable_attribute.attr("__doc__")[-1] != "\n" ? "\n\n" : "\n");
         }
         else {
             mutable_attribute.attr("__doc__") = "";
@@ -516,6 +531,25 @@ void function::add_to_namespace(
         mutable_attribute.attr("__doc__") += str("\n    ").join(make_tuple(
           "C++ signature:", f->signature(true)));
     }
+    */
+    str _doc;
+
+    if (docstring_options::show_py_signatures_)
+    {
+        _doc += str(const_cast<const char*>(detail::py_signature_tag));
+    }
+    if (doc != 0 && docstring_options::show_user_defined_)
+        _doc += doc;
+
+    if (docstring_options::show_cpp_signatures_)
+    {
+        _doc += str(const_cast<const char*>(detail::cpp_signature_tag));
+    }
+    if(_doc)
+    {    
+        object mutable_attribute(attribute);
+        mutable_attribute.attr("__doc__")= _doc;
+    }
 }
 
 BOOST_PYTHON_DECL void add_to_namespace(
@@ -590,7 +624,10 @@ extern "C"
     static PyObject* function_get_doc(PyObject* op, void*)
     {
         function* f = downcast<function>(op);
-        return python::incref(f->doc().ptr());
+        list signatures = function_doc_signature_generator::function_doc_signatures(f);
+        if(!signatures) return python::detail::none();
+        signatures.reverse();
+        return python::incref( str("\n").join(signatures).ptr());
     }
     
     static int function_set_doc(PyObject* op, PyObject* doc, void*)
@@ -619,18 +656,18 @@ extern "C"
 }
 
 static PyGetSetDef function_getsetlist[] = {
-    {"__name__", (getter)function_get_name, 0, 0, 0 },
-    {"func_name", (getter)function_get_name, 0, 0, 0 },
-    {"__class__", (getter)function_get_class, 0, 0, 0 },    // see note above
-    {"__doc__", (getter)function_get_doc, (setter)function_set_doc, 0, 0},
-    {"func_doc", (getter)function_get_doc, (setter)function_set_doc, 0, 0},
+    {const_cast<char*>("__name__"), (getter)function_get_name, 0, 0, 0 },
+    {const_cast<char*>("func_name"), (getter)function_get_name, 0, 0, 0 },
+    {const_cast<char*>("__class__"), (getter)function_get_class, 0, 0, 0 },    // see note above
+    {const_cast<char*>("__doc__"), (getter)function_get_doc, (setter)function_set_doc, 0, 0},
+    {const_cast<char*>("func_doc"), (getter)function_get_doc, (setter)function_set_doc, 0, 0},
     {NULL, 0, 0, 0, 0} /* Sentinel */
 };
 
 PyTypeObject function_type = {
     PyObject_HEAD_INIT(0)
     0,
-    "Boost.Python.function",
+    const_cast<char*>("Boost.Python.function"),
     sizeof(function),
     0,
     (destructor)function_dealloc,               /* tp_dealloc */
@@ -716,7 +753,8 @@ namespace detail
   }
   void BOOST_PYTHON_DECL pure_virtual_called()
   {
-      PyErr_SetString(PyExc_RuntimeError, "Pure virtual function called");
+      PyErr_SetString(
+          PyExc_RuntimeError, const_cast<char*>("Pure virtual function called"));
       throw_error_already_set();
   }
 }
