@@ -15,6 +15,8 @@
 #pragma warning(disable:4127)
 #endif
 
+void test_simple_repeats2();
+
 void test_simple_repeats()
 {
    using namespace boost::regex_constants;
@@ -37,6 +39,65 @@ void test_simple_repeats()
    TEST_REGEX_SEARCH("\\*", perl, "*", match_default, make_array(0, 1, -2, -2));
    TEST_REGEX_SEARCH("(ab)*", perl, "abab", match_default, make_array(0, 4, 2, 4, -2, 4, 4, -2, -2));
 
+   TEST_INVALID_REGEX("(*)", perl);
+   TEST_INVALID_REGEX("(*)", boost::regex::extended);
+   TEST_INVALID_REGEX("\\(*\\)", basic);
+   TEST_INVALID_REGEX("^*", perl);
+   TEST_INVALID_REGEX("^*", boost::regex::extended);
+   TEST_INVALID_REGEX("$*", perl);
+   TEST_INVALID_REGEX("$*", boost::regex::extended);
+   TEST_INVALID_REGEX("$*", basic);
+   TEST_INVALID_REGEX("\\b*", perl);
+   TEST_INVALID_REGEX("\\B*", perl);
+   TEST_INVALID_REGEX("\\A*", perl);
+   TEST_INVALID_REGEX("\\z*", perl);
+   TEST_INVALID_REGEX("\\Z*", perl);
+   TEST_INVALID_REGEX("\\A*", perl);
+   TEST_INVALID_REGEX("a|*", perl);
+   TEST_INVALID_REGEX("a|*", boost::regex::extended);
+   TEST_INVALID_REGEX("(+)", perl);
+   TEST_INVALID_REGEX("(+)", boost::regex::extended);
+   TEST_INVALID_REGEX("^+", perl);
+   TEST_INVALID_REGEX("^+", boost::regex::extended);
+   TEST_INVALID_REGEX("$+", perl);
+   TEST_INVALID_REGEX("$+", boost::regex::extended);
+   TEST_INVALID_REGEX("\\b+", perl);
+   TEST_INVALID_REGEX("\\B+", perl);
+   TEST_INVALID_REGEX("\\A+", perl);
+   TEST_INVALID_REGEX("\\z+", perl);
+   TEST_INVALID_REGEX("\\Z+", perl);
+   TEST_INVALID_REGEX("\\A+", perl);
+   TEST_INVALID_REGEX("a|+", perl);
+   TEST_INVALID_REGEX("a|+", boost::regex::extended);
+   TEST_INVALID_REGEX("(?)", perl);
+   TEST_INVALID_REGEX("(?)", boost::regex::extended);
+   TEST_INVALID_REGEX("^?", perl);
+   TEST_INVALID_REGEX("^?", boost::regex::extended);
+   TEST_INVALID_REGEX("$?", perl);
+   TEST_INVALID_REGEX("$?", boost::regex::extended);
+   TEST_INVALID_REGEX("\\b?", perl);
+   TEST_INVALID_REGEX("\\B?", perl);
+   TEST_INVALID_REGEX("\\A?", perl);
+   TEST_INVALID_REGEX("\\z?", perl);
+   TEST_INVALID_REGEX("\\Z?", perl);
+   TEST_INVALID_REGEX("\\A?", perl);
+   TEST_INVALID_REGEX("a|?", perl);
+   TEST_INVALID_REGEX("a|?", boost::regex::extended);
+   TEST_INVALID_REGEX("({1,2})", perl);
+   TEST_INVALID_REGEX("({1,2})", boost::regex::extended);
+   TEST_INVALID_REGEX("^{1,2}", perl);
+   TEST_INVALID_REGEX("^{1,2}", boost::regex::extended);
+   TEST_INVALID_REGEX("${1,2}", perl);
+   TEST_INVALID_REGEX("${1,2}", boost::regex::extended);
+   TEST_INVALID_REGEX("\\b{1,2}", perl);
+   TEST_INVALID_REGEX("\\B{1,2}", perl);
+   TEST_INVALID_REGEX("\\A{1,2}", perl);
+   TEST_INVALID_REGEX("\\z{1,2}", perl);
+   TEST_INVALID_REGEX("\\Z{1,2}", perl);
+   TEST_INVALID_REGEX("\\A{1,2}", perl);
+   TEST_INVALID_REGEX("a|{1,2}", perl);
+   TEST_INVALID_REGEX("a|{1,2}", boost::regex::extended);
+
    // now try operator + :
    TEST_REGEX_SEARCH("ab+", perl, "a", match_default, make_array(-2, -2));
    TEST_REGEX_SEARCH("ab+", perl, "ab", match_default, make_array(0, 2, -2, -2));
@@ -98,6 +159,25 @@ void test_simple_repeats()
    TEST_REGEX_SEARCH("a{ 2 , }", perl, "aaaaa", match_default, make_array(0, 5, -2, -2));
    TEST_REGEX_SEARCH("a{ 2 }", perl, "aaa", match_default, make_array(0, 2, -2, -2));
    TEST_REGEX_SEARCH("a\\{\\}", perl, "a{}", match_default, make_array(0, 3, -2, -2));
+   
+   TEST_REGEX_SEARCH("a{2,4}?", perl, "a", match_default, make_array(-2, -2));
+   TEST_REGEX_SEARCH("a{2,4}?", perl, "aa", match_default, make_array(0, 2, -2, -2));
+   TEST_REGEX_SEARCH("a{2,4}?", perl, "aaa", match_default, make_array(0, 2, -2, -2));
+   TEST_REGEX_SEARCH("a{2,4}?", perl, "aaaa", match_default, make_array(0, 2, -2, 2, 4, -2, -2));
+   TEST_REGEX_SEARCH("a{2,4}?", perl, "aaaaa", match_default, make_array(0, 2, -2, 2, 4, -2, -2));
+   TEST_REGEX_SEARCH("a{2,4}?$", perl, "aa", match_default, make_array(0, 2, -2, -2));
+   TEST_REGEX_SEARCH("a{2,4}?$", perl, "aaa", match_default, make_array(0, 3, -2, -2));
+   TEST_REGEX_SEARCH("a{2,4}?$", perl, "aaaa", match_default, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("a{2,4}?$", perl, "aaaaa", match_default, make_array(1, 5, -2, -2));
+   TEST_REGEX_SEARCH("^a{0,1}?$", perl, "aaaaa", match_default, make_array(-2, -2));
+   TEST_REGEX_SEARCH("^(?:a){0,1}?$", perl, "aaaaa", match_default, make_array(-2, -2));
+   TEST_REGEX_SEARCH("^a(?:bc)?", perl, "abcbc", match_any|match_all, make_array(-2, -2));
+}
+
+void test_simple_repeats2()
+{
+   using namespace boost::regex_constants;
+
    TEST_INVALID_REGEX("a{}", perl);
    TEST_INVALID_REGEX("a{", perl);
    TEST_INVALID_REGEX("a{1", perl);
@@ -341,5 +421,59 @@ void test_fast_repeats2()
    TEST_REGEX_SEARCH("ab[_[.ae.]]{2,5}?xy", perl, "ab______xy", match_default, make_array(-2, -2));
    TEST_REGEX_SEARCH("ab[_[.ae.]]{2,5}xy", perl, "ab_xy", match_default, make_array(-2, -2));
    TEST_REGEX_SEARCH("([5[.ae.]]*?).somesite", perl, "//555.somesite", match_default, make_array(2, 14, 2, 5, -2, -2));
+
+   TEST_REGEX_SEARCH("A[^B]*?B.*?C", perl, "AxBxxxx", match_default|match_partial, make_array(0, 7, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?B.*?C", perl, "AxBx", match_default|match_partial, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?B.*?C", perl, "AxBxxxx", match_default|match_partial|match_not_dot_null, make_array(0, 7, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?B.*?C", perl, "AxBx", match_default|match_partial|match_not_dot_null, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?Bx*?C", perl, "AxBxxxx", match_default|match_partial|match_not_dot_null, make_array(0, 7, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?Bx*?C", perl, "AxBx", match_default|match_partial|match_not_dot_null, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?B[xac]*?C", perl, "AxBxxxx", match_default|match_partial|match_not_dot_null, make_array(0, 7, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?B[xac]*?C", perl, "AxBx", match_default|match_partial|match_not_dot_null, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?B[xac[.ae.]]*?C", perl, "AxBxxxx", match_default|match_partial|match_not_dot_null, make_array(0, 7, -2, -2));
+   TEST_REGEX_SEARCH("A[^B]*?B[xac[.ae.]]*?C", perl, "AxBx", match_default|match_partial|match_not_dot_null, make_array(0, 4, -2, -2));
+
+
+}
+
+void test_pocessive_repeats()
+{
+   using namespace boost::regex_constants;
+   // and again for sets:
+   TEST_REGEX_SEARCH("^(\\w++|\\s++)*$", perl, "now is the time for all good men to come to the aid of the party", match_default, make_array(0, 64, 59, 64, -2, -2));
+   TEST_REGEX_SEARCH("^(\\w++|\\s++)*$", perl, "this is not a line with only words and spaces!", match_default, make_array(-2, -2));
+   TEST_REGEX_SEARCH("(\\d++)(\\w)", perl, "12345a", match_default, make_array(0, 6, 0, 5, 5, 6, -2, -2));
+   TEST_REGEX_SEARCH("(\\d++)(\\w)", perl, "12345+", match_default, make_array(-2, -2));
+   TEST_REGEX_SEARCH("(\\d++)(\\w)", perl, "12345", match_default, make_array(-2, -2));
+   TEST_REGEX_SEARCH("a++b", perl, "aaab", match_default, make_array(0, 4, -2, -2));
+   TEST_REGEX_SEARCH("(a++b)", perl, "aaab", match_default, make_array(0, 4, 0, 4, -2, -2));
+   TEST_REGEX_SEARCH("([^()]++|\\([^()]*\\))+", perl, "((abc(ade)ufh()()x", match_default, make_array(2, 18, 17, 18, -2, -2));
+   TEST_REGEX_SEARCH("\\(([^()]++|\\([^()]+\\))+\\)", perl, "(abc)", match_default, make_array(0, 5, 1, 4, -2, -2));
+   TEST_REGEX_SEARCH("\\(([^()]++|\\([^()]+\\))+\\)", perl, "(abc(def)xyz)", match_default, make_array(0, 13, 9, 12, -2, -2));
+   TEST_REGEX_SEARCH("\\(([^()]++|\\([^()]+\\))+\\)", perl, "((()aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa", match_default, make_array(-2, -2));
+   /*
+   TEST_REGEX_SEARCH("< (?: (?(R) \\d++  | [^<>]*+) | (?R)) * >", perl|mod_x, "<>", match_default, make_array(0, 2, -2, -2));
+   TEST_REGEX_SEARCH("< (?: (?(R) \\d++  | [^<>]*+) | (?R)) * >", perl|mod_x, "<abcd>", match_default, make_array(0, 6, -2, -2));
+   TEST_REGEX_SEARCH("< (?: (?(R) \\d++  | [^<>]*+) | (?R)) * >", perl|mod_x, "<abc <123> hij>", match_default, make_array(0, 15, -2, -2));
+   TEST_REGEX_SEARCH("< (?: (?(R) \\d++  | [^<>]*+) | (?R)) * >", perl|mod_x, "<abc <def> hij>", match_default, make_array(5, 10, -2, -2));
+   TEST_REGEX_SEARCH("< (?: (?(R) \\d++  | [^<>]*+) | (?R)) * >", perl|mod_x, "<abc<>def>", match_default, make_array(0, 10, -2, -2));
+   TEST_REGEX_SEARCH("< (?: (?(R) \\d++  | [^<>]*+) | (?R)) * >", perl|mod_x, "<abc<>", match_default, make_array(4, 6, -2, -2));
+   TEST_REGEX_SEARCH("< (?: (?(R) \\d++  | [^<>]*+) | (?R)) * >", perl|mod_x, "<abc", match_default, make_array(-2, -2));
+
+   TEST_REGEX_SEARCH("((< (?: (?(R) \d++  | [^<>]*+) | (?2)) * >))", perl|mod_x, "<>", match_default, make_array(0, 2, 0, 2, 0, 2, -2, -2));
+   TEST_REGEX_SEARCH("((< (?: (?(R) \d++  | [^<>]*+) | (?2)) * >))", perl|mod_x, "<abcd>", match_default, make_array(0, 6, 0, 6, 0, 6, -2, -2));
+   TEST_REGEX_SEARCH("((< (?: (?(R) \d++  | [^<>]*+) | (?2)) * >))", perl|mod_x, "<abc <123> hij>", match_default, make_array(0, 15, 0, 15, 0, 15, -2, -2));
+   TEST_REGEX_SEARCH("((< (?: (?(R) \d++  | [^<>]*+) | (?2)) * >))", perl|mod_x, "<abc <def> hij>", match_default, make_array(5, 10, 5, 10, 5, 10, -2, -2));
+   TEST_REGEX_SEARCH("((< (?: (?(R) \d++  | [^<>]*+) | (?2)) * >))", perl|mod_x, "<abc<>def>", match_default, make_array(0, 10, 0, 10, 0, 10, -2, -2));
+   TEST_REGEX_SEARCH("((< (?: (?(R) \d++  | [^<>]*+) | (?2)) * >))", perl|mod_x, "<abc<>", match_default, make_array(4, 6, 4, 6, 4, 6, -2, -2));
+   TEST_REGEX_SEARCH("((< (?: (?(R) \d++  | [^<>]*+) | (?2)) * >))", perl|mod_x, "<abc", match_default, make_array(-2, -2));
+   */
+   TEST_REGEX_SEARCH("x*+\\w", perl, "xxxxx", match_default, make_array(-2, -2));
+   TEST_REGEX_SEARCH("x*+\\w", perl, "xxxxxa", match_default, make_array(0, 6, -2, -2));
+   TEST_REGEX_SEARCH("x{1,6}+\\w", perl, "xxxxx", match_default, make_array(-2, -2));
+   TEST_REGEX_SEARCH("x{1,6}+\\w", perl, "xxxxxa", match_default, make_array(0, 6, -2, -2));
+   TEST_REGEX_SEARCH("x{1,5}+\\w", perl, "xxxxxa", match_default, make_array(0, 6, -2, -2));
+   TEST_REGEX_SEARCH("x{1,4}+\\w", perl, "xxxxxa", match_default, make_array(0, 5, -2, -2));
+   TEST_REGEX_SEARCH("x{1,3}+\\w", perl, "xxxxxa", match_default, make_array(0, 4, -2, 4, 6, -2, -2));
 }
 
