@@ -51,6 +51,7 @@ class basic_iarchive_impl {
     friend class basic_iarchive;
     version_type m_archive_library_version;
     unsigned int m_flags;
+    const basic_iarchive * m_this;
 
     //////////////////////////////////////////////////////////////////////
     // information about each serialized object loaded
@@ -161,9 +162,10 @@ class basic_iarchive_impl {
     const basic_iserializer * pending_bis;
     version_type pending_version;
 
-    basic_iarchive_impl(unsigned int flags) :
+    basic_iarchive_impl(const basic_iarchive * parent, unsigned int flags) :
         m_archive_library_version(BOOST_ARCHIVE_VERSION()),
         m_flags(flags),
+        m_this(parent),
         moveable_objects_start(0),
         moveable_objects_end(0),
         moveable_objects_recent(0),
@@ -212,17 +214,14 @@ class basic_iarchive_impl {
     const basic_pointer_iserializer * load_pointer(
         basic_iarchive & ar,
         void * & t, 
-        const basic_pointer_iserializer * bpis,
-        const basic_pointer_iserializer * (*finder)(
-            const boost::serialization::extended_type_info & type
-        )
+        const basic_pointer_iserializer * bpis
     );
 };
 
 inline void 
 basic_iarchive_impl::reset_object_address(
     const void * new_address, 
-    const void * old_address
+    const void *old_address
 ){
     // this code handles a couple of situations.
     // a) where reset_object_address is applied to an untracked object.
@@ -396,10 +395,7 @@ inline const basic_pointer_iserializer *
 basic_iarchive_impl::load_pointer(
     basic_iarchive &ar,
     void * & t,
-    const basic_pointer_iserializer * bpis_ptr,
-    const basic_pointer_iserializer * (*finder)(
-        const boost::serialization::extended_type_info & type_
-    )
+    const basic_pointer_iserializer * bpis_ptr
 ){
     class_id_type cid;
     load(ar, cid);
@@ -427,7 +423,7 @@ basic_iarchive_impl::load_pointer(
                 boost::serialization::throw_exception(
                     archive_exception(archive_exception::unregistered_class)
                 );
-            bpis_ptr = (*finder)(*eti);
+            bpis_ptr = m_this->find(*eti);
         }
         assert(NULL != bpis_ptr);
         class_id_type new_cid = register_type(bpis_ptr->get_basic_serializer());
@@ -503,7 +499,7 @@ basic_iarchive::next_object_pointer(void *t){
 
 BOOST_ARCHIVE_DECL(BOOST_PP_EMPTY())
 basic_iarchive::basic_iarchive(unsigned int flags) : 
-    pimpl(new basic_iarchive_impl(flags))
+    pimpl(new basic_iarchive_impl(this, flags))
 {}
 
 BOOST_ARCHIVE_DECL(BOOST_PP_EMPTY())
@@ -537,12 +533,9 @@ basic_iarchive::load_object(
 BOOST_ARCHIVE_DECL(const basic_pointer_iserializer *)
 basic_iarchive::load_pointer(
     void * &t, 
-    const basic_pointer_iserializer * bpis_ptr,
-    const basic_pointer_iserializer * (*finder)(
-        const boost::serialization::extended_type_info & type_
-    )
+    const basic_pointer_iserializer * bpis_ptr
 ){
-    return pimpl->load_pointer(*this, t, bpis_ptr, finder);
+    return pimpl->load_pointer(*this, t, bpis_ptr);
 }
 
 BOOST_ARCHIVE_DECL(void)
