@@ -1,5 +1,5 @@
 /////////1/////////2/////////3/////////4/////////5/////////6/////////7/////////8
-// polymorphic_text_oarchive.cpp:
+// polymorphic_oarchive.cpp:
 
 // (C) Copyright 2002 Robert Ramey - http://www.rrsd.com . 
 // Use, modification and distribution is subject to the Boost Software
@@ -12,17 +12,20 @@
 #  pragma warning (disable : 4786) // too long name, harmless warning
 #endif
 
-#include <ostream>
+#define BOOST_ARCHIVE_SOURCE
+#include <boost/archive/detail/archive_serializer_map.hpp>
+#define BOOST_ARCHIVE_SOURCE
+#include <boost/archive/detail/archive_serializer_map.hpp>
 
+#include <boost/archive/impl/archive_serializer_map.ipp>
 #include <boost/archive/polymorphic_oarchive.hpp>
 
-// explicitly instantiate for this type of text stream
-#include <boost/archive/impl/archive_pointer_oserializer.ipp>
-
 namespace boost {
 namespace archive {
+namespace detail {
 
-template class detail::archive_pointer_oserializer<polymorphic_oarchive> ;
+template class archive_serializer_map<polymorphic_oarchive>;
 
-} // namespace serialization
-} // namespace boost
+} // detail
+} // archive
+} // boost
