@@ -8,9 +8,10 @@
 //
 //  See http://www.boost.org for updates, documentation, and revision history.
 
+#include <cstddef> // NULL
+#include <cstdio> // remove
 #include <fstream>
 
-#include <cstdio> // remove
 #include <boost/config.hpp>
 #if defined(BOOST_NO_STDC_NAMESPACE)
 namespace std{ 
@@ -24,8 +25,6 @@ namespace std{
 #include <boost/serialization/nvp.hpp>
 
 #include "test_tools.hpp"
-#include <boost/preprocessor/stringize.hpp>
-#include BOOST_PP_STRINGIZE(BOOST_ARCHIVE_TEST)
 
 #include <boost/serialization/export.hpp>
 
@@ -49,7 +48,6 @@ public:
     virtual ~A(){--count;}   // default destructor
 };
 
-//BOOST_BROKEN_COMPILER_TYPE_TRAITS_SPECIALIZATION(A)
 BOOST_SERIALIZATION_SHARED_PTR(A)
 
 // B is a subclass of A
@@ -77,14 +75,14 @@ int A::count = 0;
 void save(const char * testfile, const boost::shared_ptr<A>& spa)
 {
     test_ostream os(testfile, TEST_STREAM_FLAGS);
-    test_oarchive oa(os);
+    test_oarchive oa(os, TEST_ARCHIVE_FLAGS);
     oa << BOOST_SERIALIZATION_NVP(spa);
 }
 
 void load(const char * testfile, boost::shared_ptr<A>& spa)
 {
     test_istream is(testfile, TEST_STREAM_FLAGS);
-    test_iarchive ia(is);
+    test_iarchive ia(is, TEST_ARCHIVE_FLAGS);
     ia >> BOOST_SERIALIZATION_NVP(spa);
 }
 
@@ -98,7 +96,7 @@ void save_and_load(boost::shared_ptr<A>& spa)
     load(testfile, spa1);
 
     BOOST_CHECK(
-        spa.get() == NULL && spa1.get() == NULL
+        (spa.get() == NULL && spa1.get() == NULL)
         || * spa == * spa1
     );
     std::remove(testfile);
@@ -110,7 +108,7 @@ void save2(
     const boost::shared_ptr<A>& second
 ){
     test_ostream os(testfile, TEST_STREAM_FLAGS);
-    test_oarchive oa(os);
+    test_oarchive oa(os, TEST_ARCHIVE_FLAGS);
     oa << BOOST_SERIALIZATION_NVP(first);
     oa << BOOST_SERIALIZATION_NVP(second);
 }
@@ -121,7 +119,7 @@ void load2(
     boost::shared_ptr<A>& second)
 {
     test_istream is(testfile, TEST_STREAM_FLAGS);
-    test_iarchive ia(is);
+    test_iarchive ia(is, TEST_ARCHIVE_FLAGS);
     ia >> BOOST_SERIALIZATION_NVP(first);
     ia >> BOOST_SERIALIZATION_NVP(second);
 }
@@ -153,7 +151,7 @@ void save3(
     boost::weak_ptr<A>& third
 ){
     test_ostream os(testfile, TEST_STREAM_FLAGS);
-    test_oarchive oa(os);
+    test_oarchive oa(os, TEST_ARCHIVE_FLAGS);
     oa << BOOST_SERIALIZATION_NVP(third);
     oa << BOOST_SERIALIZATION_NVP(first);
     oa << BOOST_SERIALIZATION_NVP(second);
@@ -166,7 +164,7 @@ void load3(
     boost::weak_ptr<A>& third
 ){
     test_istream is(testfile, TEST_STREAM_FLAGS);
-    test_iarchive ia(is);
+    test_iarchive ia(is, TEST_ARCHIVE_FLAGS);
     // note that we serialize the weak pointer first
     ia >> BOOST_SERIALIZATION_NVP(third);
     // inorder to test that a temporarily solitary weak pointer
