@@ -2,7 +2,7 @@
     Boost.Wave: A Standard compliant C++ preprocessor library
     http://www.boost.org/
 
-    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2009 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -14,6 +14,7 @@
 
 // include boost
 #include <boost/config.hpp>
+#include <boost/wave.hpp>
 #include <boost/filesystem/path.hpp>
 #include <boost/filesystem/operations.hpp>
 
@@ -37,6 +38,8 @@ namespace fs = boost::filesystem;
 //  level 3:    prints the expected and real result for failed tests
 //  level 4:    prints the outcome of every test
 //  level 5:    prints the real result even for succeeded tests
+//  level 6:    prints the real hooks information recorded, even for succeeded 
+//              tests
 //
 //  level 9:    prints information about almost everything
 //
@@ -59,11 +62,13 @@ main(int argc, char *argv[])
             ("copyright,c", "print out the copyright statement")
             ("config-file", po::value<std::vector<std::string> >()->composing(), 
                 "specify a config file (alternatively: @arg)")
+            ("hooks", po::value<bool>()->default_value(true), 
+                "test preprocessing hooks")
             ("debug,d", po::value<int>(), "set the debug level (0...9)")
         ;
 
     // Hidden options, will be used in in config file analysis to allow to
-    // recognise positional arguments, will not be shown to the user.
+    // recognize positional arguments, will not be shown to the user.
         po::options_description desc_hidden("Hidden options");
         desc_hidden.add_options()
             ("input", po::value<std::vector<std::string> >()->composing(), 
@@ -172,31 +177,35 @@ main(int argc, char *argv[])
                     for (std::vector<std::string>::const_iterator iit = infiles.begin(); 
                          iit != iend; ++iit)
                     {
-                    // correct the file name (prepend the cfg file path)
+                    // correct the file name (pre-pend the config file path)
                         fs::path cfgpath = fs::complete(
-                            fs::path(*cit, fs::native), fs::current_path());
+                            boost::wave::util::create_path(*cit), 
+                            boost::wave::util::current_path());
                         fs::path filepath = 
-                            cfgpath.branch_path() / fs::path(*iit, fs::native);
+                            boost::wave::util::branch_path(cfgpath) / 
+                                boost::wave::util::create_path(*iit);
                         
                         if (9 == app.get_debuglevel()) {
                             std::cerr << std::string(79, '-') << std::endl;
                             std::cerr << "executing test: " 
-                                      << filepath.native_file_string()
+                                      << boost::wave::util::native_file_string(filepath)
                                       << std::endl;
                         }
                     
                     // execute this unit test case
-                        if (!app.test_a_file(filepath.native_file_string())) {
+                        if (!app.test_a_file(
+                              boost::wave::util::native_file_string(filepath))) 
+                        {
                             if (9 == app.get_debuglevel()) {
                                 std::cerr << "failed to execute test: " 
-                                          << filepath.native_file_string()
+                                          << boost::wave::util::native_file_string(filepath)
                                           << std::endl;
                             }
                             ++error_count;
                         }
                         else if (9 == app.get_debuglevel()) {
                             std::cerr << "succeeded to execute test: " 
-                                      << filepath.native_file_string()
+                                      << boost::wave::util::native_file_string(filepath)
                                       << std::endl;
                         }
                         ++input_count;
@@ -227,26 +236,27 @@ main(int argc, char *argv[])
         for (std::vector<po::option>::const_iterator arg = arguments.begin();
              arg != arg_end; ++arg)
         {
-            fs::path filepath((*arg).value[0], fs::native);
+            fs::path filepath(boost::wave::util::create_path((*arg).value[0]));
 
             if (9 == app.get_debuglevel()) {
                 std::cerr << std::string(79, '-') << std::endl;
                 std::cerr << "executing test: " 
-                          << filepath.native_file_string()
+                          << boost::wave::util::native_file_string(filepath)
                           << std::endl;
             }
                     
-            if (!app.test_a_file(filepath.native_file_string())) {
+            if (!app.test_a_file(boost::wave::util::native_file_string(filepath))) 
+            {
                 if (9 == app.get_debuglevel()) {
                     std::cerr << "failed to execute test: " 
-                              << filepath.native_file_string()
+                              << boost::wave::util::native_file_string(filepath)
                               << std::endl;
                 }
                 ++error_count;
             }
             else if (9 == app.get_debuglevel()) {
                 std::cerr << "succeeded to execute test: " 
-                          << filepath.native_file_string()
+                          << boost::wave::util::native_file_string(filepath)
                           << std::endl;
             }
 
