@@ -12,6 +12,7 @@
 #include <functional>
 #include <boost/bind.hpp>
 #include <boost/filesystem/convenience.hpp>
+#include <boost/filesystem/fstream.hpp>
 #include <boost/lexical_cast.hpp>
 #include "./actions.hpp"
 #include "./utils.hpp"
@@ -26,7 +27,7 @@ namespace quickbook
     // Handles line-breaks (DEPRECATED!!!)
     void break_action::operator()(iterator first, iterator) const
     {
-        boost::spirit::file_position const pos = first.get_position();
+        boost::spirit::classic::file_position const pos = first.get_position();
         detail::outwarn(pos.file,pos.line) << "in column:" << pos.column << ", "
             << "[br] and \\n are deprecated" << ".\n";
         phrase << break_mark;
@@ -34,9 +35,10 @@ namespace quickbook
 
     void error_action::operator()(iterator first, iterator /*last*/) const
     {
-        boost::spirit::file_position const pos = first.get_position();
+        boost::spirit::classic::file_position const pos = first.get_position();
         detail::outerr(pos.file,pos.line)
             << "Syntax Error near column " << pos.column << ".\n";
+        ++error_count;
     }
 
     void phrase_action::operator()(iterator first, iterator last) const
@@ -223,11 +225,12 @@ namespace quickbook
 
         if (mark != list_marks.top().first) // new_indent == list_indent
         {
-            boost::spirit::file_position const pos = first.get_position();
+            boost::spirit::classic::file_position const pos = first.get_position();
             detail::outerr(pos.file,pos.line)
                 << "Illegal change of list style near column " << pos.column << ".\n";
             detail::outwarn(pos.file,pos.line)
                 << "Ignoring change of list style" << std::endl;
+            ++error_count;
         }
     }
 
@@ -239,9 +242,20 @@ namespace quickbook
         out << "</phrase>";
     }
 
-    void unexpected_char::operator()(char) const
+    void unexpected_char::operator()(iterator first, iterator last) const
     {
-        out << '#'; // print out an unexpected character
+        boost::spirit::classic::file_position const pos = first.get_position();
+
+        detail::outwarn(pos.file, pos.line)
+            << "in column:" << pos.column
+            << ", unexpected character: " << std::string(first, last)
+            << "\n";
+
+        // print out an unexpected character
+        out << "<phrase role=\"error\">";
+        while (first != last)
+            detail::print_char(*first++, out.get());
+        out << "</phrase>";
     }
 
     void anchor_action::operator()(iterator first, iterator last) const
@@ -294,12 +308,35 @@ namespace quickbook
         out << escape_actions.phrase.str();
         escape_actions.phrase.pop(); // restore the stream
     }
+    
+    std::string syntax_highlight::operator()(iterator first, iterator last) const
+    {
+        // print the code with syntax coloring
+        if (source_mode == "c++")
+        {
+            parse(first, last, cpp_p);
+        }
+        else if (source_mode == "python")
+        {
+            parse(first, last, python_p);
+        }
+        else if (source_mode == "teletype")
+        {
+            parse(first, last, teletype_p);
+        }
+        else
+        {
+            BOOST_ASSERT(0);
+        }
+
+        std::string str;
+        temp.swap(str);
+        
+        return str;
+    }
 
     void code_action::operator()(iterator first, iterator last) const
     {
-        std::string save;
-        phrase.swap(save);
-
         // preprocess the code section to remove the initial indentation
         std::string program(first, last);
         detail::unindent(program);
@@ -310,21 +347,19 @@ namespace quickbook
         iterator last_(program.end(), program.end());
         first_.set_position(first.get_position());
 
-        // print the code with syntax coloring
-        if (source_mode == "c++")
-        {
-            parse(first_, last_, cpp_p);
-        }
-        else if (source_mode == "python")
-        {
-            parse(first_, last_, python_p);
-        }
-
-        std::string str;
-        temp.swap(str);
+        std::string save;
         phrase.swap(save);
 
-        out << "<programlisting>\n";
+        // print the code with syntax coloring
+        std::string str = syntax_p(first_, last_);
+
+        phrase.swap(save);
+
+        //
+        // We must not place a \n after the <programlisting> tag
+        // otherwise PDF output starts code blocks with a blank line:
+        //
+        out << "<programlisting>";
         out << str;
         out << "</programlisting>\n";
     }
@@ -335,16 +370,8 @@ namespace quickbook
         out.swap(save);
 
         // print the code with syntax coloring
-        if (source_mode == "c++")
-        {
-            parse(first, last, cpp_p);
-        }
-        else if (source_mode == "python")
-        {
-            parse(first, last, python_p);
-        }
-        std::string str;
-        temp.swap(str);
+        std::string str = syntax_p(first, last);
+
         out.swap(save);
 
         out << "<code>";
@@ -376,9 +403,73 @@ namespace quickbook
     {
         fs::path const img_path(std::string(first, last));
 
+        std::string attr_text;
+        if(fs::extension(img_path) == ".svg")
+        {
+           //
+           // SVG's need special handling:
+           //
+           // 1) We must set the "format" attribute, otherwise
+           //    HTML generation produces code that will not display
+           //    the image at all.
+           // 2) We need to set the "contentwidth" and "contentdepth"
+           //    attributes, otherwise the image will be displayed inside
+           //    a tiny box with scrollbars (Firefox), or else cropped to
+           //    fit in a tiny box (IE7).
+           //
+           attr_text = " format=\"SVG\"";
+           //
+           // Image paths are relative to the html subdirectory:
+           //
+           fs::path img;
+           if(img_path.root_path().empty())
+              img = "html" / img_path;  // relative path
+           else
+              img = img_path;   // absolute path
+           //
+           // Now load the SVG file:
+           //
+           std::string svg_text;
+           fs::ifstream fs(img);
+           char c;
+           while(fs.get(c) && fs.good())
+              svg_text.push_back(c);
+           //
+           // Extract the svg header from the file:
+           //
+           std::string::size_type a, b;
+           a = svg_text.find("<svg");
+           b = svg_text.find('>', a);
+           svg_text = (a == std::string::npos) ? "" : svg_text.substr(a, b - a);
+           //
+           // Now locate the "width" and "height" attributes
+           // and borrow their values:
+           //
+           a = svg_text.find("width");
+           a = svg_text.find('=', a);
+           a = svg_text.find('\"', a);
+           b = svg_text.find('\"', a + 1);
+           if(a != std::string::npos)
+           {
+              attr_text.append(" contentwidth=");
+              attr_text.append(svg_text.begin() + a, svg_text.begin() + b + 1);
+           }
+           a = svg_text.find("height");
+           a = svg_text.find('=', a);
+           a = svg_text.find('\"', a);
+           b = svg_text.find('\"', a + 1);
+           if(a != std::string::npos)
+           {
+              attr_text.append(" contentdepth=");
+              attr_text.append(svg_text.begin() + a, svg_text.begin() + b + 1);
+           }
+        }
+
         phrase << "<inlinemediaobject>";
 
-        phrase << "<imageobject><imagedata fileref=\"";
+        phrase << "<imageobject><imagedata ";
+        phrase << attr_text;
+        phrase << " fileref=\"";
         while (first != last)
             detail::print_char(*first++, phrase.get());
         phrase << "\"></imagedata></imageobject>";
@@ -412,9 +503,10 @@ namespace quickbook
         BOOST_ASSERT(actions.template_info.size());
         if (actions.templates.find_top_scope(actions.template_info[0]))
         {
-            boost::spirit::file_position const pos = first.get_position();
+            boost::spirit::classic::file_position const pos = first.get_position();
             detail::outerr(pos.file,pos.line)
                 << "Template Redefinition: " << actions.template_info[0] << std::endl;
+            ++actions.error_count;
         }
 
         actions.template_info.push_back(std::string(first, last));
@@ -429,12 +521,12 @@ namespace quickbook
         bool break_arguments(
             std::vector<std::string>& template_info
           , std::vector<std::string> const& template_
-          , boost::spirit::file_position const& pos
+          , boost::spirit::classic::file_position const& pos
         )
         {
             if (template_.size()-1 != template_info.size())
             {
-                while (template_.size()-1 != template_info.size())
+                while (template_.size()-1 > template_info.size())
                 {
                     // Try to break the last argument at the first space found
                     // and push it into the back of template_info. Do this
@@ -447,6 +539,8 @@ namespace quickbook
                         break;
                     std::string first(str.begin(), str.begin()+l_pos);
                     std::string::size_type r_pos = str.find_first_not_of(" \t\r\n", l_pos);
+                    if (r_pos == std::string::npos)
+                        break;
                     std::string second(str.begin()+r_pos, str.end());
                     str = first;
                     template_info.push_back(second);
@@ -471,7 +565,7 @@ namespace quickbook
         get_arguments(
             std::vector<std::string>& template_info
           , std::vector<std::string> const& template_
-          , boost::spirit::file_position const& pos
+          , boost::spirit::classic::file_position const& pos
           , quickbook::actions& actions
         )
         {
@@ -486,10 +580,11 @@ namespace quickbook
                 tinfo.push_back(*arg);
                 template_symbol template_(tinfo, pos);
 
-                if (template_symbol* p = actions.templates.find_top_scope(*tpl))
+                if (actions.templates.find_top_scope(*tpl))
                 {
                     detail::outerr(pos.file,pos.line)
                         << "Duplicate Symbol Found" << std::endl;
+                    ++actions.error_count;
                     return std::make_pair(false, tpl);
                 }
                 else
@@ -504,7 +599,7 @@ namespace quickbook
         bool parse_template(
             std::string& body
           , std::string& result
-          , boost::spirit::file_position const& template_pos
+          , boost::spirit::classic::file_position const& template_pos
           , quickbook::actions& actions
         )
         {
@@ -535,7 +630,7 @@ namespace quickbook
                 iterator first(body.begin(), body.end(), actions.filename.native_file_string().c_str());
                 first.set_position(template_pos);
                 iterator last(body.end(), body.end());
-                r = boost::spirit::parse(first, last, phrase_p).full;
+                r = boost::spirit::classic::parse(first, last, phrase_p).full;
                 actions.phrase.swap(result);
             }
             else
@@ -550,7 +645,7 @@ namespace quickbook
                 iterator first(iter, body.end(), actions.filename.native_file_string().c_str());
                 first.set_position(template_pos);
                 iterator last(body.end(), body.end());
-                r = boost::spirit::parse(first, last, block_p).full;
+                r = boost::spirit::classic::parse(first, last, block_p).full;
                 actions.out.swap(result);
             }
             return r;
@@ -559,13 +654,14 @@ namespace quickbook
 
     void do_template_action::operator()(iterator first, iterator) const
     {
-        boost::spirit::file_position const pos = first.get_position();
+        boost::spirit::classic::file_position const pos = first.get_position();
         ++actions.template_depth;
         if (actions.template_depth > actions.max_template_depth)
         {
             detail::outerr(pos.file,pos.line)
                 << "Infinite loop detected" << std::endl;
             --actions.template_depth;
+            ++actions.error_count;
             return;
         }
 
@@ -577,7 +673,7 @@ namespace quickbook
             BOOST_ASSERT(symbol);
 
             std::vector<std::string> template_ = boost::get<0>(*symbol);
-            boost::spirit::file_position template_pos = boost::get<1>(*symbol);
+            boost::spirit::classic::file_position template_pos = boost::get<1>(*symbol);
 
             std::vector<std::string> template_info;
             std::swap(template_info, actions.template_info);
@@ -588,6 +684,7 @@ namespace quickbook
             {
                 actions.pop(); // restore the actions' states
                 --actions.template_depth;
+                ++actions.error_count;
                 return;
             }
 
@@ -613,11 +710,16 @@ namespace quickbook
 
             if (!parse_template(body, result, template_pos, actions))
             {
-                boost::spirit::file_position const pos = first.get_position();
+                boost::spirit::classic::file_position const pos = first.get_position();
                 detail::outerr(pos.file,pos.line)
-                    << "Expanding template" << std::endl;
+                    << "Expanding template:" << template_info[0] << std::endl
+                    << "------------------begin------------------" << std::endl
+                    << body
+                    << "------------------end--------------------" << std::endl
+                    << std::endl;
                 actions.pop(); // restore the actions' states
                 --actions.template_depth;
+                ++actions.error_count;
                 return;
             }
         }
@@ -812,9 +914,11 @@ namespace quickbook
         --section_level;
         if (section_level < 0)
         {
-            boost::spirit::file_position const pos = first.get_position();
+            boost::spirit::classic::file_position const pos = first.get_position();
             detail::outerr(pos.file,pos.line)
                 << "Mismatched [endsect] near column " << pos.column << ".\n";
+            ++error_count;
+            
             // $$$ TODO: somehow fail parse else BOOST_ASSERT(std::string::npos != n)
             // $$$ below will assert.
         }
@@ -840,10 +944,13 @@ namespace quickbook
             if(!fs::equivalent(outtmp /= *out, temp /= *file))
                 break;
         }
-        std::divides<fs::path> concat;
         out = (out == outdir.begin()) ? outdir.end() : out;
-        temp = std::accumulate(out, outdir.end(), fs::path(), boost::bind(concat, _1, ".."));
-        return std::accumulate(file, path.end(), temp, concat);
+
+        fs::path result = fs::path();
+        for(; out != outdir.end(); ++out)
+            if(*out != ".") result /= "..";
+        std::divides<fs::path> concat;
+        return std::accumulate(file, path.end(), result, concat);
     }
 
     fs::path calculate_relative_path(
@@ -870,7 +977,7 @@ namespace quickbook
         out << "\" />\n";
     }
 
-    void cpp_code_snippet_grammar::pass_thru(iterator first, iterator last) const
+    void code_snippet_actions::pass_thru(iterator first, iterator last)
     {
         code += *first;
     }
@@ -880,7 +987,7 @@ namespace quickbook
         int callout_id = 0;
     }
 
-    void cpp_code_snippet_grammar::callout(iterator first, iterator last, char const* role) const
+    void code_snippet_actions::callout(iterator first, iterator last, char const* role)
     {
         using detail::callout_id;
         code += "``'''";
@@ -895,24 +1002,26 @@ namespace quickbook
         callouts.push_back(std::string(first, last));
     }
 
-    void cpp_code_snippet_grammar::inline_callout(iterator first, iterator last) const
+    void code_snippet_actions::inline_callout(iterator first, iterator last)
     {
         callout(first, last, "callout_bug");
     }
 
-    void cpp_code_snippet_grammar::line_callout(iterator first, iterator last) const
+    void code_snippet_actions::line_callout(iterator first, iterator last)
     {
         callout(first, last, "line_callout_bug");
     }
 
-    void cpp_code_snippet_grammar::escaped_comment(iterator first, iterator last) const
+    void code_snippet_actions::escaped_comment(iterator first, iterator last)
     {
         if (!code.empty())
         {
             detail::unindent(code); // remove all indents
             if (code.size() != 0)
             {
-                snippet += "\n\n``\n" + code + "``\n\n";
+                snippet += "\n\n";
+                snippet += source_type;
+                snippet += "``\n" + code + "``\n\n";
                 code.clear();
             }
         }
@@ -924,7 +1033,7 @@ namespace quickbook
         }
     }
 
-    void cpp_code_snippet_grammar::compile(iterator first, iterator last) const
+    void code_snippet_actions::compile(iterator first, iterator last)
     {
         using detail::callout_id;
         if (!code.empty())
@@ -932,7 +1041,9 @@ namespace quickbook
             detail::unindent(code); // remove all indents
             if (code.size() != 0)
             {
-                snippet += "\n\n```\n" + code + "```\n\n";
+                snippet += "\n\n";
+                snippet += source_type;
+                snippet += "```\n" + code + "```\n\n";
             }
 
             if(callouts.size() > 0)
@@ -967,7 +1078,7 @@ namespace quickbook
         id.clear();
     }
 
-    void load_snippets(
+    int load_snippets(
         std::string const& file
       , std::vector<template_symbol>& storage   // snippets are stored in a
                                                 // vector of template_symbols
@@ -977,14 +1088,25 @@ namespace quickbook
         std::string code;
         int err = detail::load(file, code);
         if (err != 0)
-            return; // return early on error
+            return err; // return early on error
 
         typedef position_iterator<std::string::const_iterator> iterator_type;
         iterator_type first(code.begin(), code.end(), file);
         iterator_type last(code.end(), code.end());
 
-        cpp_code_snippet_grammar g(storage, doc_id);
-        boost::spirit::parse(first, last, g);
+        size_t fname_len = file.size();
+        bool is_python = fname_len >= 3
+            && file[--fname_len]=='y' && file[--fname_len]=='p' && file[--fname_len]=='.';
+        code_snippet_actions a(storage, doc_id, is_python ? "[python]" : "[c++]");
+        // TODO: Should I check that parse succeeded?
+        if(is_python) {
+            boost::spirit::classic::parse(first, last, python_code_snippet_grammar(a));
+        }
+        else {
+            boost::spirit::classic::parse(first, last, cpp_code_snippet_grammar(a));
+        }
+
+        return 0;
     }
 
     namespace
@@ -992,7 +1114,7 @@ namespace quickbook
         fs::path include_search(fs::path const & current, std::string const & name)
         {
             fs::path path(name,fs::native);
-            
+
             // If the path is relative, try and resolve it.
             if (!path.is_complete())
             {
@@ -1001,7 +1123,7 @@ namespace quickbook
                 {
                     return current / path;
                 }
-                
+
                 // Search in each of the include path locations.
                 BOOST_FOREACH(std::string const & p, include_path)
                 {
@@ -1013,7 +1135,7 @@ namespace quickbook
                     }
                 }
             }
-            
+
             return path;
         }
     }
@@ -1023,16 +1145,18 @@ namespace quickbook
         fs::path path = include_search(actions.filename.branch_path(), std::string(first,last));
         std::string ext = fs::extension(path);
         std::vector<template_symbol> storage;
-        load_snippets(path.string(), storage, ext, actions.doc_id);
+        actions.error_count +=
+            load_snippets(path.string(), storage, ext, actions.doc_id);
 
         BOOST_FOREACH(template_symbol const& ts, storage)
         {
             std::string tname = boost::get<0>(ts)[0];
             if (actions.templates.find_top_scope(tname))
             {
-                boost::spirit::file_position const pos = boost::get<1>(ts);
+                boost::spirit::classic::file_position const pos = boost::get<1>(ts);
                 detail::outerr(pos.file, pos.line)
                     << "Template Redefinition: " << tname << std::endl;
+                ++actions.error_count;
             }
             else
             {
@@ -1069,7 +1193,7 @@ namespace quickbook
         }
 
         // update the __FILENAME__ macro
-        *boost::spirit::find(actions.macro, "__FILENAME__") = actions.filename.native_file_string();
+        *boost::spirit::classic::find(actions.macro, "__FILENAME__") = actions.filename.native_file_string();
 
         // parse the file
         quickbook::parse(actions.filename.native_file_string().c_str(), actions, true);
@@ -1116,6 +1240,9 @@ namespace quickbook
         out << "      <year>" << year << "</year>\n";
     }
 
+    static void write_document_title(collector& out, quickbook::actions& actions);
+    static void write_document_info(collector& out, quickbook::actions& actions);
+
     void pre(collector& out, quickbook::actions& actions, bool ignore_docinfo)
     {
         // The doc_info in the file has been parsed. Here's what we'll do
@@ -1125,7 +1252,7 @@ namespace quickbook
             actions.doc_id = detail::make_identifier(
                 actions.doc_title.begin(),actions.doc_title.end());
 
-        if (actions.doc_dirname.empty())
+        if (actions.doc_dirname.empty() && actions.doc_type == "library")
             actions.doc_dirname = actions.doc_id;
 
         if (actions.doc_last_revision.empty())
@@ -1168,12 +1295,58 @@ namespace quickbook
             << "<!DOCTYPE library PUBLIC \"-//Boost//DTD BoostBook XML V1.0//EN\"\n"
             << "     \"http://www.boost.org/tools/boostbook/dtd/boostbook.dtd\">\n"
             << '<' << actions.doc_type << "\n"
-            << "    id=\"" << actions.doc_id << "\"\n"
-            << "    name=\"" << actions.doc_title << "\"\n"
-            << "    dirname=\"" << actions.doc_dirname << "\"\n"
-            << "    last-revision=\"" << actions.doc_last_revision << "\" \n"
-            << "    xmlns:xi=\"http://www.w3.org/2001/XInclude\">\n"
-            << "  <" << actions.doc_type << "info>\n";
+            << "    id=\"" << actions.doc_id << "\"\n";
+        
+        if(actions.doc_type == "library")
+        {
+            out << "    name=\"" << actions.doc_title << "\"\n";
+        }
+
+        if(!actions.doc_dirname.empty())
+        {
+            out << "    dirname=\"" << actions.doc_dirname << "\"\n";
+        }
+
+        out << "    last-revision=\"" << actions.doc_last_revision << "\" \n"
+            << "    xmlns:xi=\"http://www.w3.org/2001/XInclude\">\n";
+            
+        if(actions.doc_type == "library") {
+            write_document_info(out, actions);
+            write_document_title(out, actions);
+        }
+        else {
+            write_document_title(out, actions);
+            write_document_info(out, actions);
+        }
+    }
+    
+    void post(collector& out, quickbook::actions& actions, bool ignore_docinfo)
+    {
+        // if we're ignoring the document info, do nothing.
+        if (ignore_docinfo)
+        {
+            return;
+        }
+
+        // We've finished generating our output. Here's what we'll do
+        // *after* everything else.
+        out << "\n</" << actions.doc_type << ">\n\n";
+    }
+
+    void write_document_title(collector& out, quickbook::actions& actions)
+    {
+        if (!actions.doc_title.empty())
+        {
+            out<< "  <title>" << actions.doc_title;
+            if (!actions.doc_version.empty())
+                out << ' ' << actions.doc_version;
+            out<< "</title>\n\n\n";
+        }
+    }
+
+    void write_document_info(collector& out, quickbook::actions& actions)
+    {
+        out << "  <" << actions.doc_type << "info>\n";
 
         if(!actions.doc_authors.empty())
         {
@@ -1232,27 +1405,6 @@ namespace quickbook
         out << "  </" << actions.doc_type << "info>\n"
             << "\n"
         ;
-
-        if (!actions.doc_title.empty())
-        {
-            out << "  <title>" << actions.doc_title;
-            if (!actions.doc_version.empty())
-                out << ' ' << actions.doc_version;
-            out << "</title>\n\n\n";
-        }
-    }
-
-    void post(collector& out, quickbook::actions& actions, bool ignore_docinfo)
-    {
-        // if we're ignoring the document info, do nothing.
-        if (ignore_docinfo)
-        {
-            return;
-        }
-
-        // We've finished generating our output. Here's what we'll do
-        // *after* everything else.
-        out << "\n</" << actions.doc_type << ">\n\n";
     }
 
     void phrase_to_string_action::operator()(iterator first, iterator last) const
