@@ -11,15 +11,15 @@
 #define BOOST_SPIRIT_QUICKBOOK_PHRASE_HPP
 
 #include "detail/utils.hpp"
-#include <boost/spirit/core.hpp>
-#include <boost/spirit/utility/confix.hpp>
-#include <boost/spirit/utility/chset.hpp>
-#include <boost/spirit/actor/assign_actor.hpp>
-#include <boost/spirit/dynamic/if.hpp>
+#include <boost/spirit/include/classic_core.hpp>
+#include <boost/spirit/include/classic_confix.hpp>
+#include <boost/spirit/include/classic_chset.hpp>
+#include <boost/spirit/include/classic_assign_actor.hpp>
+#include <boost/spirit/include/classic_if.hpp>
 
 namespace quickbook
 {
-    using namespace boost::spirit;
+    using namespace boost::spirit::classic;
 
     template <typename Rule, typename Action>
     inline void
@@ -145,6 +145,7 @@ namespace quickbook
                         >> !(hard_space
                             >> template_args)
                     ) )
+                    >> eps_p(']')
                     ;
 
                 brackets =
@@ -224,6 +225,7 @@ namespace quickbook
                         |   macroref
                         |   headerref
                         |   conceptref
+                        |   globalref
                         |   bold
                         |   italic
                         |   underline
@@ -354,6 +356,15 @@ namespace quickbook
                         )                               [actions.conceptref_post]
                     ;
 
+                globalref =
+                    "globalref" >> hard_space
+                    >>  (*(anychar_p -
+                            (']' | hard_space)))        [actions.globalref_pre]
+                    >>  (   eps_p(']')
+                        |   (hard_space >> phrase)
+                        )                               [actions.globalref_post]
+                    ;
+
                 bold =
                         ch_p('*')                       [actions.bold_pre]
                     >>  blank >> phrase                 [actions.bold_post]
@@ -393,6 +404,7 @@ namespace quickbook
                     (
                         str_p("c++")
                     |   "python"
+                    |   "teletype"
                     )                                   [assign_a(actions.source_mode)]
                     ;
 
@@ -404,8 +416,8 @@ namespace quickbook
 
             rule<Scanner>   space, blank, comment, phrase, phrase_markup, image,
                             phrase_end, bold, italic, underline, teletype,
-                            strikethrough, escape, url, common, funcref,
-                            classref, memberref, enumref, macroref, headerref, conceptref,
+                            strikethrough, escape, url, common, funcref, classref,
+                            memberref, enumref, macroref, headerref, conceptref, globalref,
                             anchor, link, hard_space, eol, inline_code, simple_format,
                             simple_bold, simple_italic, simple_underline,
                             simple_teletype, source_mode, template_, template_arg,
