@@ -111,7 +111,7 @@ namespace
             for(itr = m_subcolumns.begin(); itr != m_subcolumns.end(); ++itr){
                 std::pair<int, int> spans;
                 spans = itr->second.get_spans();
-                rows = std::max(rows, spans.first);
+                rows = (std::max)(rows, spans.first);
                 cols += spans.second;
             }
             ++rows;
@@ -130,7 +130,7 @@ namespace
             if(fs::is_directory(*itr)){
                 std::pair<col_node::subcolumns_t::iterator, bool> result 
                     = node.m_subcolumns.insert(
-                        std::make_pair(itr->leaf(), col_node())
+                        std::make_pair(itr->filename(), col_node())
                     );
                 build_node_tree(*itr, result.first->second);
             }
@@ -629,7 +629,7 @@ namespace
             if(! fs::is_directory(*itr))
                 continue;
             
-            string test_name = itr->leaf();
+            string test_name = itr->filename();
             // strip off the ".test" is there is one
             string::size_type s = test_name.find( ".test" );
             if(string::npos != s)
@@ -661,7 +661,7 @@ namespace
         col_node::subcolumns_t::const_iterator itr;
         int max_depth = 1;
         for(itr = root.m_subcolumns.begin(); itr != root.m_subcolumns.end(); ++itr){
-            max_depth = std::max(max_depth, itr->second.rows);
+            max_depth = (std::max)(max_depth, itr->second.rows);
         }
         return max_depth;
     }
@@ -728,7 +728,7 @@ namespace
                 throw std::string("binary path not found");
             if(*leaf_itr != "libs")
                 --leaf_itr;
-            test_lib_dir.remove_leaf();
+            test_lib_dir.remove_filename();
         }
 
         if(leaf_itr == fs::initial_path().end())
@@ -768,7 +768,7 @@ namespace
             }
             if(boost_root.empty())
                 throw std::string("boost root not found");
-            boost_root.remove_leaf();
+            boost_root.remove_filename();
         }
 
         return boost_root;
@@ -856,9 +856,9 @@ int cpp_main( int argc, char * argv[] ) // note name!
             "  options: --compiler name     Run for named compiler only\n"
             "           --ignore-pass       Do not report tests which pass all compilers\n"
             "           --no-warn           Warnings not reported if test passes\n"
-		    "           --boost-root path default derived from current path.\n"
-		    "           --locate-root path  Path to ALL_LOCATE_TARGET for bjam;\n"
-		    "                               default boost-root.\n"
+            "           --boost-root path default derived from current path.\n"
+            "           --locate-root path  Path to ALL_LOCATE_TARGET for bjam;\n"
+            "                               default boost-root.\n"
             "           --comment path      Path to file containing HTML\n"
             "                               to be copied into status-file.\n"
             "           --notes path        Path to file containing HTML\n"
@@ -873,10 +873,10 @@ int cpp_main( int argc, char * argv[] ) // note name!
         return 1;
     }
 
-	if(boost_root.empty())
-		boost_root = find_boost_root();
-	if ( locate_root.empty() ) 
-		locate_root = boost_root;
+    if(boost_root.empty())
+        boost_root = find_boost_root();
+    if ( locate_root.empty() ) 
+        locate_root = boost_root;
 
     report.open( fs::path( argv[1], fs::native ) );
     if ( !report )
@@ -888,7 +888,7 @@ int cpp_main( int argc, char * argv[] ) // note name!
     if ( argc == 3 )
     {
         fs::path links_path( argv[2], fs::native );
-        links_name = links_path.leaf();
+        links_name = links_path.filename();
         links_file.open( links_path );
         if ( !links_file )
         {
