@@ -3,7 +3,7 @@
 
     http://www.boost.org/
 
-    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2009 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -76,9 +76,8 @@ typedef boost::archive::text_oarchive oarchive;
 
 ///////////////////////////////////////////////////////////////////////////////
 //  Import required names
-using namespace boost::spirit;
+using namespace boost::spirit::classic;
 
-using std::string;
 using std::pair;
 using std::vector;
 using std::getline;
@@ -113,11 +112,11 @@ using std::istreambuf_iterator;
 
 ///////////////////////////////////////////////////////////////////////////////
 // print the current version
-string get_version()
+std::string get_version()
 {
-    string version (context_type::get_version_string());
+    std::string version (context_type::get_version_string());
     version = version.substr(1, version.size()-2);      // strip quotes
-    version += string(" (" CPP_VERSION_DATE_STR ")");   // add date
+    version += std::string(" (" CPP_VERSION_DATE_STR ")");   // add date
     return version;
 }
 
@@ -139,7 +138,7 @@ int print_copyright()
         "Wave: A Standard conformant C++ preprocessor based on the Boost.Wave library",
         "http://www.boost.org/",
         "",
-        "Copyright (c) 2001-2008 Hartmut Kaiser, Distributed under the Boost",
+        "Copyright (c) 2001-2009 Hartmut Kaiser, Distributed under the Boost",
         "Software License, Version 1.0. (See accompanying file",
         "LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)",
         0
@@ -176,13 +175,13 @@ namespace cmd_line_utils {
 
     // Additional command line parser which interprets '@something' as an 
     // option "config-file" with the value "something".
-    inline pair<string, string> 
-    at_option_parser(string const&s)
+    inline pair<std::string, std::string> 
+    at_option_parser(std::string const&s)
     {
         if ('@' == s[0]) 
-            return std::make_pair(string("config-file"), s.substr(1));
+            return std::make_pair(std::string("config-file"), s.substr(1));
         else
-            return pair<string, string>();
+            return pair<std::string, std::string>();
     }
 
     // class, which keeps include file information read from the command line
@@ -190,13 +189,13 @@ namespace cmd_line_utils {
     public:
         include_paths() : seen_separator(false) {}
 
-        vector<string> paths;       // stores user paths
-        vector<string> syspaths;    // stores system paths
+        vector<std::string> paths;       // stores user paths
+        vector<std::string> syspaths;    // stores system paths
         bool seen_separator;        // command line contains a '-I-' option
 
         // Function which validates additional tokens from command line.
         static void 
-        validate(boost::any &v, vector<string> const &tokens)
+        validate(boost::any &v, vector<std::string> const &tokens)
         {
             if (v.empty())
                 v = boost::any(include_paths());
@@ -205,7 +204,7 @@ namespace cmd_line_utils {
 
             BOOST_ASSERT(p);
             // Assume only one path per '-I' occurrence.
-            string const& t = po::validators::get_single_string(tokens);
+            std::string const& t = po::validators::get_single_string(tokens);
             if (t == "-") {
             // found -I- option, so switch behaviour
                 p->seen_separator = true;
@@ -223,7 +222,7 @@ namespace cmd_line_utils {
 
     // Read all options from a given config file, parse and add them to the
     // given variables_map
-    bool read_config_file_options(string const &filename, 
+    bool read_config_file_options(std::string const &filename, 
         po::options_description const &desc, po::variables_map &vm,
         bool may_fail = false)
     {
@@ -238,20 +237,20 @@ namespace cmd_line_utils {
             return false;
         }
         
-    vector<string> options;
-    string line;
+    vector<std::string> options;
+    std::string line;
 
         while (std::getline(ifs, line)) {
         // skip empty lines
-            string::size_type pos = line.find_first_not_of(" \t");
-            if (pos == string::npos) 
+            std::string::size_type pos = line.find_first_not_of(" \t");
+            if (pos == std::string::npos) 
                 continue;
             
         // skip comment lines
             if ('#' != line[pos]) {
             // strip leading and trailing whitespace
-                string::size_type endpos = line.find_last_not_of(" \t");
-                BOOST_ASSERT(endpos != string::npos);
+                std::string::size_type endpos = line.find_last_not_of(" \t");
+                BOOST_ASSERT(endpos != std::string::npos);
                 options.push_back(line.substr(pos, endpos-pos+1));
             }
         }
@@ -454,9 +453,10 @@ namespace {
 #if BOOST_WAVE_SERIALIZATION != 0
         try {
             if (vm.count("state") > 0) {
-                fs::path state_file (vm["state"].as<string>(), fs::native);
+                fs::path state_file (
+                    boost::wave::util::create_path(vm["state"].as<std::string>()));
                 if (state_file == "-") 
-                    state_file = fs::path("wave.state", fs::native);
+                    state_file = boost::wave::util::create_path("wave.state");
 
                 std::ios::openmode mode = std::ios::in;
 
@@ -467,8 +467,8 @@ namespace {
                 if (ifs.is_open()) {
                     using namespace boost::serialization;
                     iarchive ia(ifs);
-                    string version;
-                    
+                    std::string version;
+
                     ia >> make_nvp("version", version);  // load version
                     if (version == CPP_VERSION_FULL_STR)
                         ia >> make_nvp("state", ctx);    // load the internal tables from disc
@@ -498,9 +498,10 @@ namespace {
 #if BOOST_WAVE_SERIALIZATION != 0
         try {
             if (vm.count("state") > 0) {
-                fs::path state_file (vm["state"].as<string>(), fs::native);
+                fs::path state_file (boost::wave::util::create_path(
+                    vm["state"].as<std::string>()));
                 if (state_file == "-") 
-                    state_file = fs::path("wave.state", fs::native);
+                    state_file = boost::wave::util::create_path("wave.state");
 
                 std::ios::openmode mode = std::ios::out;
 
@@ -516,7 +517,7 @@ namespace {
                 else {
                     using namespace boost::serialization;
                     oarchive oa(ofs);
-                    string version(CPP_VERSION_FULL_STR);
+                    std::string version(CPP_VERSION_FULL_STR);
                     oa << make_nvp("version", version);  // write version
                     oa << make_nvp("state", ctx);        // write the internal tables to disc
                 }
@@ -535,11 +536,11 @@ namespace {
     {
     // open file for macro names listing
         std::ofstream macronames_out;
-        fs::path macronames_file (filename, fs::native);
+        fs::path macronames_file (boost::wave::util::create_path(filename));
 
         if (macronames_file != "-") {
             macronames_file = fs::complete(macronames_file);
-            fs::create_directories(macronames_file.branch_path());
+            fs::create_directories(boost::wave::util::branch_path(macronames_file));
             macronames_out.open(macronames_file.string().c_str());
             if (!macronames_out.is_open()) {
                 cerr << "wave: could not open file for macro name listing: " 
@@ -568,7 +569,7 @@ namespace {
             
             if (ctx.get_macro_definition(*it, has_pars, predef, pos, pars, def))
             {
-                macronames_out << *it;
+                macronames_out << (predef ? "-P" : "-D") << *it;
                 if (has_pars) {
                 // list the parameter names for function style macros
                     macronames_out << "(";
@@ -592,10 +593,6 @@ namespace {
                     macronames_out << (*dit).get_value();
                 }
 
-            // predefined macros get a 'P' appended
-                if (predef) 
-                    macronames_out << " (P)";
-                    
                 macronames_out << std::endl;
             }
         }
@@ -618,19 +615,19 @@ int error_count = 0;
 
     try {
     // process the given file
-    string instring;
+    std::string instring;
 
         instream.unsetf(std::ios::skipws);
 
         if (!input_is_stdin) {
 #if defined(BOOST_NO_TEMPLATED_ITERATOR_CONSTRUCTORS)
             // this is known to be very slow for large files on some systems
-            copy (istream_iterator<char>(instream),
-                  istream_iterator<char>(), 
-                  inserter(instring, instring.end()));
+            copy (std::istream_iterator<char>(instream),
+                  std::istream_iterator<char>(), 
+                  std::inserter(instring, instring.end()));
 #else
-            instring = string(istreambuf_iterator<char>(instream.rdbuf()),
-                              istreambuf_iterator<char>());
+            instring = std::string(std::istreambuf_iterator<char>(instream.rdbuf()),
+                                   std::istreambuf_iterator<char>());
 #endif 
         }
         
@@ -644,10 +641,11 @@ int error_count = 0;
     
         if (vm.count("traceto")) {
         // try to open the file, where to put the trace output
-        fs::path trace_file (vm["traceto"].as<string>(), fs::native);
+        fs::path trace_file (boost::wave::util::create_path(
+            vm["traceto"].as<std::string>()));
         
             if (trace_file != "-") {
-                fs::create_directories(trace_file.branch_path());
+                fs::create_directories(boost::wave::util::branch_path(trace_file));
                 traceout.open(trace_file.string().c_str());
                 if (!traceout.is_open()) {
                     cerr << "wave: could not open trace file: " << trace_file 
@@ -667,10 +665,11 @@ int error_count = 0;
     // Open the stream where to output the list of included file names
         if (vm.count("listincludes")) {
         // try to open the file, where to put the include list 
-        fs::path includes_file(vm["listincludes"].as<string>(), fs::native);
+        fs::path includes_file(boost::wave::util::create_path(
+            vm["listincludes"].as<std::string>()));
         
             if (includes_file != "-") {
-                fs::create_directories(includes_file.branch_path());
+                fs::create_directories(boost::wave::util::branch_path(includes_file));
                 includelistout.open(includes_file.string().c_str());
                 if (!includelistout.is_open()) {
                     cerr << "wave: could not open include list file: " 
@@ -721,8 +720,8 @@ int error_count = 0;
     // This this the central piece of the Wave library, it provides you with 
     // the iterators to get the preprocessed tokens and allows to configure
     // the preprocessing stage in advance.
-    bool allow_output = true;   // will be manipulated from inside the hooks object
-    string default_outfile;     // will be used from inside the hooks object
+    bool allow_output = true;     // will be manipulated from inside the hooks object
+    std::string default_outfile;  // will be used from inside the hooks object
     trace_macro_expansion<token_type> hooks(preserve_whitespace, 
         output, traceout, includelistout, enable_trace, enable_system_command,
         allow_output, default_outfile);
@@ -800,10 +799,10 @@ int error_count = 0;
 
     // add include directories to the system include search paths
         if (vm.count("sysinclude")) {
-        vector<string> syspaths = vm["sysinclude"].as<vector<string> >();
-        
-            vector<string>::const_iterator end = syspaths.end();
-            for (vector<string>::const_iterator cit = syspaths.begin(); 
+          vector<std::string> syspaths = vm["sysinclude"].as<vector<std::string> >();
+
+            vector<std::string>::const_iterator end = syspaths.end();
+            for (vector<std::string>::const_iterator cit = syspaths.begin(); 
                  cit != end; ++cit)
             {
                 ctx.add_sysinclude_path(cmd_line_utils::trim_quotes(*cit).c_str());
@@ -814,9 +813,9 @@ int error_count = 0;
         if (vm.count("include")) {
             cmd_line_utils::include_paths const &ip = 
                 vm["include"].as<cmd_line_utils::include_paths>();
-            vector<string>::const_iterator end = ip.paths.end();
+            vector<std::string>::const_iterator end = ip.paths.end();
 
-            for (vector<string>::const_iterator cit = ip.paths.begin(); 
+            for (vector<std::string>::const_iterator cit = ip.paths.begin(); 
                  cit != end; ++cit)
             {
                 ctx.add_include_path(cmd_line_utils::trim_quotes(*cit).c_str());
@@ -827,8 +826,8 @@ int error_count = 0;
                 ctx.set_sysinclude_delimiter();
                  
         // add system include directories to the include path
-            vector<string>::const_iterator sysend = ip.syspaths.end();
-            for (vector<string>::const_iterator syscit = ip.syspaths.begin(); 
+            vector<std::string>::const_iterator sysend = ip.syspaths.end();
+            for (vector<std::string>::const_iterator syscit = ip.syspaths.begin(); 
                  syscit != sysend; ++syscit)
             {
                 ctx.add_sysinclude_path(cmd_line_utils::trim_quotes(*syscit).c_str());
@@ -837,9 +836,9 @@ int error_count = 0;
     
     // add additional defined macros 
         if (vm.count("define")) {
-            vector<string> const &macros = vm["define"].as<vector<string> >();
-            vector<string>::const_iterator end = macros.end();
-            for (vector<string>::const_iterator cit = macros.begin(); 
+          vector<std::string> const &macros = vm["define"].as<vector<std::string> >();
+            vector<std::string>::const_iterator end = macros.end();
+            for (vector<std::string>::const_iterator cit = macros.begin(); 
                  cit != end; ++cit)
             {
                 ctx.add_macro_definition(*cit);
@@ -848,10 +847,10 @@ int error_count = 0;
 
     // add additional predefined macros 
         if (vm.count("predefine")) {
-            vector<string> const &predefmacros = 
-                vm["predefine"].as<vector<string> >();
-            vector<string>::const_iterator end = predefmacros.end();
-            for (vector<string>::const_iterator cit = predefmacros.begin(); 
+            vector<std::string> const &predefmacros = 
+                vm["predefine"].as<vector<std::string> >();
+            vector<std::string>::const_iterator end = predefmacros.end();
+            for (vector<std::string>::const_iterator cit = predefmacros.begin(); 
                  cit != end; ++cit)
             {
                 ctx.add_macro_definition(*cit, true);
@@ -860,13 +859,13 @@ int error_count = 0;
 
     // undefine specified macros
         if (vm.count("undefine")) {
-            vector<string> const &undefmacros = 
-                vm["undefine"].as<vector<string> >();
-            vector<string>::const_iterator end = undefmacros.end();
-            for (vector<string>::const_iterator cit = undefmacros.begin(); 
+            vector<std::string> const &undefmacros = 
+                vm["undefine"].as<vector<std::string> >();
+            vector<std::string>::const_iterator end = undefmacros.end();
+            for (vector<std::string>::const_iterator cit = undefmacros.begin(); 
                  cit != end; ++cit)
             {
-                ctx.remove_macro_definition((*cit).c_str(), true);
+                ctx.remove_macro_definition(*cit, true);
             }
         }
 
@@ -884,7 +883,8 @@ int error_count = 0;
     // open the output file
         if (vm.count("output")) {
         // try to open the file, where to put the preprocessed output
-        fs::path out_file (vm["output"].as<string>(), fs::native);
+        fs::path out_file (boost::wave::util::create_path(
+            vm["output"].as<std::string>()));
 
             if (out_file == "-") {
                 allow_output = false;     // inhibit output initially
@@ -892,7 +892,7 @@ int error_count = 0;
             }
             else {
                 out_file = fs::complete(out_file);
-                fs::create_directories(out_file.branch_path());
+                fs::create_directories(boost::wave::util::branch_path(out_file));
                 output.open(out_file.string().c_str());
                 if (!output.is_open()) {
                     cerr << "wave: could not open output file: " 
@@ -904,15 +904,15 @@ int error_count = 0;
         }
         else if (!input_is_stdin && vm.count("autooutput")) {
         // generate output in the file <input_base_name>.i
-        fs::path out_file (file_name, fs::native);
-        std::string basename (out_file.leaf());
+        fs::path out_file (boost::wave::util::create_path(file_name));
+        std::string basename (boost::wave::util::leaf(out_file));
         std::string::size_type pos = basename.find_last_of(".");
         
             if (std::string::npos != pos)
                 basename = basename.substr(0, pos);
-            out_file = out_file.branch_path() / (basename + ".i");
+            out_file = boost::wave::util::branch_path(out_file) / (basename + ".i");
 
-            fs::create_directories(out_file.branch_path());
+            fs::create_directories(boost::wave::util::branch_path(out_file));
             output.open(out_file.string().c_str());
             if (!output.is_open()) {
                 cerr << "wave: could not open output file: " 
@@ -941,13 +941,13 @@ int error_count = 0;
         // add the filenames to force as include files in _reverse_ order
         // the second parameter 'is_last' of the force_include function should
         // be set to true for the last (first given) file.
-            vector<string> const &force = 
-                vm["forceinclude"].as<vector<string> >();
-            vector<string>::const_reverse_iterator rend = force.rend();
-            for (vector<string>::const_reverse_iterator cit = force.rbegin(); 
+            vector<std::string> const &force = 
+                vm["forceinclude"].as<vector<std::string> >();
+            vector<std::string>::const_reverse_iterator rend = force.rend();
+            for (vector<std::string>::const_reverse_iterator cit = force.rbegin(); 
                  cit != rend; /**/)
             {
-                string filename(*cit);
+                std::string filename(*cit);
                 first.force_include(filename.c_str(), ++cit == rend);
             }
         }
@@ -981,8 +981,15 @@ int error_count = 0;
                 first = ctx.begin(instring.begin(), instring.end());
             }
             
+        bool need_to_advanve = false;
+        
             do {
                 try {
+                    if (need_to_advanve) {
+                        ++first;
+                        need_to_advanve = false;
+                    }
+                        
                     while (first != last) {
                     // store the last known good token position
                         current_position = (*first).get_position();
@@ -1010,7 +1017,7 @@ int error_count = 0;
                 // some preprocessing error
                     if (is_interactive || boost::wave::is_recoverable(e)) {
                         error_count += report_error_message(ctx, e);
-                        ++first;    // advance to the next token
+                        need_to_advanve = true;   // advance to the next token
                     }
                     else {
                         throw;      // re-throw for non-recoverable errors
@@ -1022,7 +1029,7 @@ int error_count = 0;
                         boost::wave::cpplexer::is_recoverable(e)) 
                     {
                         error_count += report_error_message(e);
-                        ++first;    // advance to the next token
+                        need_to_advanve = true;   // advance to the next token
                     }
                     else {
                         throw;      // re-throw for non-recoverable errors
@@ -1036,7 +1043,7 @@ int error_count = 0;
 
     // list all defined macros at the end of the preprocessing
         if (vm.count("macronames")) {
-            if (!list_macro_names(ctx, vm["macronames"].as<string>()))
+            if (!list_macro_names(ctx, vm["macronames"].as<std::string>()))
                 return -1;
         }
     }
@@ -1090,7 +1097,7 @@ main (int argc, char *argv[])
             ("help,h", "print out program usage (this message)")
             ("version,v", "print the version number")
             ("copyright,c", "print out the copyright statement")
-            ("config-file", po::value<vector<string> >()->composing(), 
+            ("config-file", po::value<vector<std::string> >()->composing(), 
                 "specify a config file (alternatively: @filepath)")
         ;
 
@@ -1098,22 +1105,22 @@ main (int argc, char *argv[])
     po::options_description desc_generic ("Options allowed additionally in a config file");
 
         desc_generic.add_options()
-            ("output,o", po::value<string>(), 
+            ("output,o", po::value<std::string>(), 
                 "specify a file [arg] to use for output instead of stdout or "
                 "disable output [-]")
             ("autooutput,E", 
                 "output goes into a file named <input_basename>.i")
             ("include,I", po::value<cmd_line_utils::include_paths>()->composing(), 
                 "specify an additional include directory")
-            ("sysinclude,S", po::value<vector<string> >()->composing(), 
+            ("sysinclude,S", po::value<vector<std::string> >()->composing(), 
                 "specify an additional system include directory")
-            ("forceinclude,F", po::value<vector<string> >()->composing(),
+            ("forceinclude,F", po::value<vector<std::string> >()->composing(),
                 "force inclusion of the given file")
-            ("define,D", po::value<vector<string> >()->composing(), 
+            ("define,D", po::value<vector<std::string> >()->composing(), 
                 "specify a macro to define (as macro[=[value]])")
-            ("predefine,P", po::value<vector<string> >()->composing(), 
+            ("predefine,P", po::value<vector<std::string> >()->composing(), 
                 "specify a macro to predefine (as macro[=[value]])")
-            ("undefine,U", po::value<vector<string> >()->composing(), 
+            ("undefine,U", po::value<vector<std::string> >()->composing(), 
                 "specify a macro to undefine")
             ("nesting,n", po::value<int>(), 
                 "specify a new maximal include nesting depth")
@@ -1122,7 +1129,7 @@ main (int argc, char *argv[])
     po::options_description desc_ext ("Extended options (allowed everywhere)");
 
         desc_ext.add_options()
-            ("traceto,t", po::value<string>(), 
+            ("traceto,t", po::value<std::string>(), 
                 "output macro expansion tracing information to a file [arg] "
                 "or to stderr [-]")
             ("timer", "output overall elapsed computing time to stderr")
@@ -1131,9 +1138,9 @@ main (int argc, char *argv[])
             ("variadics", "enable certain C99 extensions in C++ mode")
             ("c99", "enable C99 mode (implies --variadics)")
 #endif 
-            ("listincludes,l", po::value<string>(), 
+            ("listincludes,l", po::value<std::string>(), 
                 "list names of included files to a file [arg] or to stdout [-]")
-            ("macronames,m", po::value<string>(), 
+            ("macronames,m", po::value<std::string>(), 
                 "list all defined macros to a file [arg] or to stdout [-]")
             ("preserve,p", po::value<int>()->default_value(0), 
                 "preserve whitespace\n"
@@ -1154,7 +1161,7 @@ main (int argc, char *argv[])
             ("noguard,G", "disable include guard detection")
 #endif
 #if BOOST_WAVE_SERIALIZATION != 0
-            ("state,s", po::value<string>(), 
+            ("state,s", po::value<std::string>(), 
                 "load and save state information from/to the given file [arg] "
                 "or 'wave.state' [-] (interactive mode only)")
 #endif
@@ -1179,7 +1186,7 @@ main (int argc, char *argv[])
 
 //     // Try to find a wave.cfg in the same directory as the executable was 
 //     // started from. If this exists, treat it as a wave config file
-//     fs::path filename(argv[0], fs::native);
+//     fs::path filename(argv[0]);
 // 
 //         filename = filename.branch_path() / "wave.cfg";
 //         cmd_line_utils::read_config_file_options(filename.string(), 
@@ -1196,8 +1203,12 @@ main (int argc, char *argv[])
     // file for all files in a certain project.
         if (arguments.size() > 0 && arguments[0].value[0] != "-") {
         // construct full path of input file
-            fs::path input_dir (fs::complete(fs::path(arguments[0].value[0], fs::native)));
-            input_dir = input_dir.normalize().branch_path();    // chop of file name
+            fs::path input_dir (fs::complete(
+                boost::wave::util::create_path(arguments[0].value[0])));
+
+        // chop of file name
+            input_dir = boost::wave::util::branch_path(
+                boost::wave::util::normalize(input_dir)); 
 
         // walk up the hierarchy, trying to find a file wave.cfg 
             while (!input_dir.empty()) {
@@ -1207,17 +1218,17 @@ main (int argc, char *argv[])
                 {
                     break;    // break on the first cfg file found
                 }
-                input_dir = input_dir.branch_path();
+                input_dir = boost::wave::util::branch_path(input_dir);
             }
         }
         
     // if there is specified at least one config file, parse it and add the 
     // options to the main variables_map
         if (vm.count("config-file")) {
-            vector<string> const &cfg_files = 
-                vm["config-file"].as<vector<string> >();
-            vector<string>::const_iterator end = cfg_files.end();
-            for (vector<string>::const_iterator cit = cfg_files.begin(); 
+            vector<std::string> const &cfg_files = 
+                vm["config-file"].as<vector<std::string> >();
+            vector<std::string>::const_iterator end = cfg_files.end();
+            for (vector<std::string>::const_iterator cit = cfg_files.begin(); 
                  cit != end; ++cit)
             {
             // parse a single config file and store the results
